import json
import os

CONFIG_DICT = {
    "CONFIG_WRAPPIXELVALUES" : False,
    "CONFIG_MEDIA_PATH" : "",
    "CONFIG_SESSION_PATH" : "",
    "CONFIG_JES4PY_PATH" : ""
    }
CONFIG_FILENAME = ".jes4pyconf"

def getConfigVal(key):
    return CONFIG_DICT[key]

def setConfigVal(key, val):
    CONFIG_DICT[key] = val
    writeDict(CONFIG_DICT)

def initPath():
    import jes4py
    CONFIG_DICT["CONFIG_JES4PY_PATH"] = os.path.dirname(jes4py.__file__)

def initDict():
    filePath = os.path.join(os.path.expanduser("~"), CONFIG_FILENAME)
    if os.path.exists(filePath):
        curDict = readDict(filePath)
        global CONFIG_DICT
        pathDict = readDict(filePath)
        CONFIG_DICT["CONFIG_SESSION_PATH"]=pathDict["CONFIG_MEDIA_PATH"]
        CONFIG_DICT["CONFIG_MEDIA_PATH"]=pathDict["CONFIG_MEDIA_PATH"]

def writeDict(dict):
    filePath = os.path.join(os.path.expanduser("~"), CONFIG_FILENAME)
    f = open(filePath, "w")
    f.write(json.dumps(dict))
    f.close()

def readDict(filePath):
    f = open(filePath, "r")
    contents = f.read()
    dict = json.loads(contents)
    f.close()
    return dict



# Media interface for JES-emulator, a Python3 implementation of JES,
# the Jython Environment for Students, that does not require Jython.
#
# This file is a heavily modified version of jes/python/media.py from the
# JES distribution:
#     # Media Wrappers for "Introduction to Media Computation"
#     # Started: Mark Guzdial, 2 July 2002
#
# 

import sys
import os
import math
# import traceback
# import user
#import pictureMod
# import Picture
# import Pixel
from jes4py.Picture import Picture
from jes4py.PixelColor import Pixel, Color
# import Sound
from jes4py.Sound import Sound
# import StoppableInput
# import StoppableOutput
# import Sample
# import Samples
from jes4py.Sample import Sample
from jes4py.Samples import Samples
# import MoviePlayer
# import MovieWriter
from jes4py import FileChooser
import random
from jes4py import Config

# from jes.tools.framesequencer import FrameSequencerTool

# import org.python.core.PyString as String

# Support a media shortcut

# if ( mediaFolder == "" ):
mediaFolder = os.getcwd() + os.sep

# Store last pickAFile() opening

_lastFilePath = ""

true = 1
false = 0


def setMediaPath(file=None):
    global mediaFolder
    if(file == None):
        FileChooser.pickMediaPath()
    else:
        FileChooser.setMediaPath(file)
    mediaFolder = getMediaPath()
    return mediaFolder


def getMediaPath(filename=""):
    return str(FileChooser.getMediaPath(filename))


def setMediaFolder(file=None):
    return setMediaPath(file)


def setTestMediaFolder():
    global mediaFolder
    mediaFolder = os.getcwd() + os.sep


def getMediaFolder(filename=""):
    return str(getMediaPath(filename))


def showMediaFolder():
    global mediaFolder
    print("The media path is currently: ", mediaFolder)


def getShortPath(filename):
    dirs = filename.split(os.sep)
    if len(dirs) < 1:
        return "."
    elif len(dirs) == 1:
        return str(dirs[0])
    else:
        return str(dirs[len(dirs) - 2] + os.sep + dirs[len(dirs) - 1])


def addLibPath(directory=None):
    if directory is None:
        directory = pickAFolder()

    if os.path.isdir(directory):
        sys.path.insert(0, directory)
    elif directory is not None:
        raise ValueError("There is no directory at " + directory)

    return directory

setLibPath = addLibPath


##
# Global sound functions
##

def samplesToSound(samples, maxIndex=100):
    # Find maxX
    maxIndex = max([getIndex(s) for s in samples])
    newSound = makeEmptySound(maxIndex + 1,
                              int(getSamplingRate(samples[0].getSound())))
    for s in samples:
        x = getIndex(s)
        setSampleValueAt(newSound, x, getSampleValue(s))
    return newSound


def makeSound(filename, maxIndex=100):
    global mediaFolder
    if not isinstance(filename, str):
        return samplesToSound(filename, maxIndex=maxIndex)
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not os.path.isfile(filename):
        print("There is no file at " + filename)
        raise ValueError
    return Sound(filename)

# MMO (1 Dec 2005): capped size of sound to 600
# Brian O (29 Apr 2008): changed first argument to be number of samples,
# added optional 2nd argument of sampling rate


def makeEmptySound(numSamples, samplingRate=Sound.SAMPLE_RATE):
    numSamples = int(numSamples)
    if numSamples <= 0 or samplingRate <= 0:
        print("makeEmptySound(numSamples[, samplingRate]): numSamples and samplingRate must each be greater than 0")
        raise ValueError
    if (numSamples / samplingRate) > 600:
        print("makeEmptySound(numSamples[, samplingRate]): Created sound must be less than 600 seconds")
        raise ValueError
    return Sound(numSamples, samplingRate)

# Brian O (5 May 2008): Added method for creating sound by duration


def makeEmptySoundBySeconds(seconds, samplingRate=Sound.SAMPLE_RATE):
    if seconds <= 0 or samplingRate <= 0:
        print("makeEmptySoundBySeconds(numSamples[, samplingRate]): numSamples and samplingRate must each be greater than 0")
        raise ValueError
    if seconds > 600:
        print("makeEmptySoundBySeconds(numSamples[, samplingRate]): Created sound must be less than 600 seconds")
        raise ValueError
    return Sound(seconds * samplingRate, samplingRate)

# PamC: Added this function to duplicate a sound


def duplicateSound(sound):
    if not isinstance(sound, Sound):
        print("duplicateSound(sound): Input is not a sound")
        raise ValueError
    return Sound(sound)


def getSamples(sound):
    if not isinstance(sound, Sound):
        print("getSamples(sound): Input is not a sound")
        raise ValueError
#    return Samples(sound)
    return Samples.getSamples(sound)


def play(sound):
    if not isinstance(sound, Sound):
        print("play(sound): Input is not a sound")
        raise ValueError
    sound.play()


def blockingPlay(sound):
    if not isinstance(sound, Sound):
        print("blockingPlay(sound): Input is not a sound")
        raise ValueError
    sound.blockingPlay()

# Buck Scharfnorth (27 May 2008): Added method for stopping play of a sound


def stopPlaying(sound):
    if not isinstance(sound, Sound):
        print("stopPlaying(sound): Input is not a sound")
        raise ValueError
    sound.stopPlaying()


def playAtRate(sound, rate):
    if not isinstance(sound, Sound):
        print("playAtRate(sound,rate): First input is not a sound")
        raise ValueError
    # sound.playAtRate(rate)
    sound.playAtRateDur(rate, sound.getLength())


def playAtRateDur(sound, rate, dur):
    if not isinstance(sound, Sound):
        print("playAtRateDur(sound,rate,dur): First input is not a sound")
        raise ValueError
    sound.playAtRateDur(rate, dur)

# 20June03 new functionality in JavaSound (ellie)


def playInRange(sound, start, stop):
    if not isinstance(sound, Sound):
        print("playInRange(sound,start,stop): First input is not a sound")
        raise ValueError
    # sound.playInRange(start,stop)
    sound.playAtRateInRange(
        1, start - Sound._SoundIndexOffset, stop - Sound._SoundIndexOffset)

# 20June03 new functionality in JavaSound (ellie)


def blockingPlayInRange(sound, start, stop):
    if not isinstance(sound, Sound):
        print("blockingPlayInRange(sound,start,stop): First input is not a sound")
        raise ValueError
    # sound.blockingPlayInRange(start,stop)
    sound.blockingPlayAtRateInRange(
        1, start - Sound._SoundIndexOffset, stop - Sound._SoundIndexOffset)

# 20June03 new functionality in JavaSound (ellie)


def playAtRateInRange(sound, rate, start, stop):
    if not isinstance(sound, Sound):
        print("playAtRateInRAnge(sound,rate,start,stop): First input is not a sound")
        raise ValueError
    sound.playAtRateInRange(
        rate, start - Sound._SoundIndexOffset, stop - Sound._SoundIndexOffset)

# 20June03 new functionality in JavaSound (ellie)


def blockingPlayAtRateInRange(sound, rate, start, stop):
    if not isinstance(sound, Sound):
        print("blockingPlayAtRateInRange(sound,rate,start,stop): First input is not a sound")
        raise ValueError
    sound.blockingPlayAtRateInRange(
        rate, start - Sound._SoundIndexOffset, stop - Sound._SoundIndexOffset)


def getSamplingRate(sound):
    if not isinstance(sound, Sound):
        print("getSamplingRate(sound): Input is not a sound")
        raise ValueError
    return sound.getSamplingRate()


def setSampleValueAt(sound, index, value):
    if not isinstance(sound, Sound):
        print("setSampleValueAt(sound,index,value): First input is not a sound")
        raise ValueError
    if index < Sound._SoundIndexOffset:
        print("You asked for the sample at index: " + str(index) + ".  This number is less than " + str(Sound._SoundIndexOffset) + ".  Please try" + " again using an index in the range [" + str(Sound._SoundIndexOffset) + "," + str(getLength(sound) - 1 + Sound._SoundIndexOffset) + "].")
        raise ValueError
    if index > getLength(sound) - 1 + Sound._SoundIndexOffset:
        print("You are trying to access the sample at index: " + str(index) + ", but the last valid index is at " + str(getLength(sound) - 1 + Sound._SoundIndexOffset))
        raise ValueError
    sound.setSampleValue(index - Sound._SoundIndexOffset, int(value))


def getSampleValueAt(sound, index):
    if not isinstance(sound, Sound):
        print("getSampleValueAt(sound,index): First input is not a sound")
        raise ValueError
    if index < Sound._SoundIndexOffset:
        print("You asked for the sample at index: " + str(index) + ".  This number is less than " + str(Sound._SoundIndexOffset) + ".  Please try" + " again using an index in the range [" + str(Sound._SoundIndexOffset) + "," + str(getLength(sound) - 1 + Sound._SoundIndexOffset) + "].")
        raise ValueError
    if index > getLength(sound) - 1 + Sound._SoundIndexOffset:
        print("You are trying to access the sample at index: " + str(index) + ", but the last valid index is at " + str(getLength(sound) - 1 + Sound._SoundIndexOffset))
        raise ValueError
    return sound.getSampleValue(index - Sound._SoundIndexOffset)


def getSampleObjectAt(sound, index):
    if not isinstance(sound, Sound):
        print("getSampleObjectAt(sound,index): First input is not a sound")
        raise ValueError
    # return sound.getSampleObjectAt(index-Sound._SoundIndexOffset)
    if index < Sound._SoundIndexOffset:
        print("You asked for the sample at index: " + str(index) + ".  This number is less than " + str(Sound._SoundIndexOffset) + ".  Please try" + " again using an index in the range [" + str(Sound._SoundIndexOffset) + "," + str(getLength(sound) - 1 + Sound._SoundIndexOffset) + "].")
        raise ValueError
    if index > getLength(sound) - 1 + Sound._SoundIndexOffset:
        print("You are trying to access the sample at index: " + str(index) + ", but the last valid index is at " + str(getLength(sound) - 1 + Sound._SoundIndexOffset))
        raise ValueError
    return Sample(sound, index - Sound._SoundIndexOffset)


def setSample(sample, value):
    if not isinstance(sample, Sample):
        print("setSample(sample,value): First input is not a sample")
        raise ValueError
    if value > 32767:
        value = 32767
    elif value < -32768:
        value = -32768
    # Need to coerce value to integer
    return sample.setValue(int(value))

# PamC: Added this function to be a better name than setSample


def setSampleValue(sample, value):
    setSample(sample, value)


def getSample(sample):
    if not isinstance(sample, Sample):
        print("getSample(sample): Input is not a sample")
        raise ValueError
    return sample.getValue()

# PamC: Added this to be a better name for getSample


def getSampleValue(sample):
    return getSample(sample)


def getSound(sample):
    if not isinstance(sample, Sample):
        print("getSound(sample): Input is not a sample")
        raise ValueError
    return sample.getSound()


def getLength(sound):
    if not isinstance(sound, Sound):
        print("getLength(sound): Input is not a sound")
        raise ValueError
    return sound.getLength()

# PamC: Added this function as a more meaningful name for getLength


def getNumSamples(sound):
    return getLength(sound)

# PamC: Added this function to return the number of seconds
# in a sound


def getDuration(sound):
    if not isinstance(sound, Sound):
        print("getDuration(sound): Input is not a sound")
        raise ValueError
    return sound.getLength() / sound.getSamplingRate()


def writeSoundTo(sound, filename):
    global mediaFolder
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not isinstance(sound, Sound):
        print("writeSoundTo(sound,filename): First input is not a sound")
        raise ValueError
    sound.writeToFile(filename)


def randomSamples(someSound, number):
    samplelist = []
    samples = getSamples(someSound)
    for count in range(number):
        samplelist.append(random.choice(samples))
    explore(samplesToSound(samplelist))


def getIndex(sample):
    return int(str(sample).split()[2])

##
# Globals for styled text
##


#def makeStyle(fontName, emph, size):
#    return awt.Font(fontName, emph, size)

# sansSerif = "SansSerif"
# serif = "Serif"
# mono = "Monospaced"
# italic = awt.Font.ITALIC
# bold = awt.Font.BOLD
# plain = awt.Font.PLAIN

##
# Global color functions
##

# Buck Scharfnorth (28 May 2008): if bool == 1 colors will be (value % 256)
#                                 if bool == 0 colors will be truncated to 0 or 255
# updated (13 May 2009):
# THIS GLOBAL FUNCTION CHANGES JES SETTINGS - this value overwrites
# the value in the JES options menu.


def setColorWrapAround(setting):
    Pixel.setWrapLevels(bool(setting))

def getColorWrapAround():
    return Pixel.getWrapLevels()

def pickAColor():
    # Dorn 5/8/2009:  Edited to be thread safe since this code is executed from an
    # interpreter JESThread and will result in an update to the main JES GUI due to
    # it being a modal dialog.
    return Color.pickAColor()

# Constants
black = Color(0, 0, 0)
white = Color(255, 255, 255)
blue = Color(0, 0, 255)
red = Color(255, 0, 0)
green = Color(0, 255, 0)
gray = Color(128, 128, 128)
darkGray = Color(64, 64, 64)
lightGray = Color(192, 192, 192)
yellow = Color(255, 255, 0)
orange = Color(255, 200, 0)
pink = Color(255, 175, 175)
magenta = Color(255, 0, 255)
cyan = Color(0, 255, 255)

##
# Global picture functions
##


def randomPixels(somePic, number):
    pixellist = []
    pixels = getPixels(somePic)
    for count in range(number):
        pixellist.append(random.choice(pixels))
    explore(pixelsToPicture(pixellist))


def pixelsToPicture(pixels, defaultColor=white, maxX=100, maxY=100):
    # Find maxX
    maxX = max([getX(p) for p in pixels])
    # find maxY
    maxY = max([getY(p) for p in pixels])
    newpic = makeEmptyPicture(maxX + 1, maxY + 1, defaultColor)
    for pixel in pixels:
        x = getX(pixel)
        y = getY(pixel)
        setColor(getPixel(newpic, x, y), getColor(pixel))
    return newpic


def makePicture(filename, defaultColor=white):
    global mediaFolder
    if not isinstance(filename, str):
        return pixelsToPicture(filename, defaultColor=defaultColor)
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not os.path.isfile(filename):
        print("makePicture(filename): There is no file at " + filename)
        raise ValueError
    picture = Picture()
    picture.loadOrFail(filename)
    return picture

# MMO (1 Dec 2005): Capped width/height to max 10000 and min 1
# alexr (6 Sep 2006): fixed to work without the Python classes.
# PamC (6 July 2007): added new optional param to allow for empty pictures
# with different background colors.


def makeEmptyPicture(width, height, acolor=white):
    if width > 10000 or height > 10000:
        print("makeEmptyPicture(width, height[, acolor]): height and width must be less than 10000 each")
        raise ValueError
    if width <= 0 or height <= 0:
        print("makeEmptyPicture(width, height[, acolor]): height and width must be greater than 0 each")
        raise ValueError
    picture = Picture(width, height, acolor)
    # picture.createImage(width, height)
    # picture.filename = ''
    # careful here; do we want empty strings or "None"?
    return picture


def getPixels(picture):
    if not isinstance(picture, Picture):
        print("getPixels(picture): Input is not a picture")
        raise ValueError
    return picture.getPixels()


def getAllPixels(picture):
    return getPixels(picture)


def getWidth(picture):
    if not isinstance(picture, Picture):
        print("getWidth(picture): Input is not a picture")
        raise ValueError
    return picture.getWidth()


def getHeight(picture):
    if not isinstance(picture, Picture):
        print("getHeight(picture): Input is not a picture")
        raise ValueError
    return picture.getHeight()


def show(picture, title=None):
    # picture.setTitle(getShortPath(picture.filename))
    # if title <> None:
            # picture.setTitle(title)
    if not isinstance(picture, Picture):
        print("show(picture): Input is not a picture")
        raise ValueError
    picture.show()

def repaint(picture):
    #if not (isinstance(picture, World) or isinstance(picture, Picture)):
    #    print("repaint(picture): Input is not a picture or a world")
    #    raise ValueError
    if not isinstance(picture, Picture):
        print("repaint(picture): Input is not a picture")
        raise ValueError
    picture.repaint()

## adding graphics to your pictures! ##


def addLine(picture, x1, y1, x2, y2, acolor=black):
    if not isinstance(picture, Picture):
        print("addLine(picture, x1, y1, x2, y2[, color]): First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addLine(picture, x1, y1, x2, y2[, color]): Last input is not a color")
        raise ValueError
    #g = picture.getBufferedImage().createGraphics()
    # g.setColor(acolor.color)
    #g.drawLine(x1 - 1,y1 - 1,x2 - 1,y2 - 1)
    picture.addLine(acolor, x1, y1, x2, y2)


def addText(picture, x, y, string, acolor=black):
    if not isinstance(picture, Picture):
        print("addText(picture, x, y, string[, color]): First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addText(picture, x, y, string[, color]): Last input is not a color")
        raise ValueError
    #g = picture.getBufferedImage().getGraphics()
    #g.setColor(acolor.color)
    #g.drawString(string, x - 1, y - 1)
    picture.addText(acolor, x, y, string)

# PamC: Added this function to allow different font styles

# def addTextWithStyle(picture, x, y, text, style, acolor=black):
#     if not isinstance(picture, Picture):
#         print("addTextWithStyle(picture, x, y, string, style[, color]): First input is not a picture")
#         raise ValueError
#     if not isinstance(style, String):
#         print("addTextWithStyle(picture, x, y, string, style[, color]): Input is not a style (see makeStyle)")
#         raise ValueError
#     if not isinstance(acolor, Color):
#         print("addTextWithStyle(picture, x, y, string, style[, color]): Last input is not a color")
#         raise ValueError
#     picture.addTextWithStyle(acolor, x, y, text, style)

# - JRS -- 2020-06-23 -- START OF MODIFICATIONS

def addRect(picture, x, y, w, h, acolor=black):
    if not isinstance(picture, Picture):
        print("addRect(picture, x, y, w, h[, color]): First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addRect(picture, x, y, w, h[, color]): Last input is not a color")
        raise ValueError
    picture.addRect(acolor, x, y, w, h)


def addRectFilled(picture, x, y, w, h, acolor=black):
    if not isinstance(picture, Picture):
        print("addRectFilled(picture, x, y, w, h[, color]): First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addRectFilled(picture, x, y, w, h[, color]): Last input is not a color")
        raise ValueError
    picture.addRectFilled(acolor, x, y, w, h)

# PamC: Added the following addOval, addOvalFilled, addArc, and addArcFilled
# functions to add more graphics to pictures.


def addOval(picture, x, y, w, h, acolor=black):
    if not isinstance(picture, Picture):
        print("addOval(picture, x, y, w, h[, color]): First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addOval(picture, x, y, w, h[, color]): Last input is not a color")
        raise ValueError
    picture.addOval(acolor, x, y, w, h)


def addOvalFilled(picture, x, y, w, h, acolor=black):
    if not isinstance(picture, Picture):
        print("addOvalFilled(picture, x, y, w, h[, color]): First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addOvalFilled(picture, x, y, w, h[, color]): Last input is not a color")
        raise ValueError
    picture.addOvalFilled(acolor, x, y, w, h)


def addArc(picture, x, y, w, h, start, angle, acolor=black):
    if not isinstance(picture, Picture):
        print("addArc(picture, x, y, w, h, start, angle[, color]): First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addArc(picture, x, y, w, h, start, angle[, color]): Last input is not a color")
        raise ValueError
    picture.addArc(acolor, x, y, w, h, start, angle)


def addArcFilled(picture, x, y, w, h, start, angle, acolor=black):
    if not isinstance(picture, Picture):
        print("addArcFilled(picture, x, y, w, h[, color]): First First input is not a picture")
        raise ValueError
    if not isinstance(acolor, Color):
        print("addArcFilled(picture, x, y, w, h, start, angle[, color]): Last input is not a color")
        raise ValueError
    picture.addArcFilled(acolor, x, y, w, h, start, angle)

# note the -1; in JES we think of pictures as starting at (1,1) but not
# in the Java.
##
# 29 Oct 2008: -1 changed to Picture._PictureIndexOffset


def getPixel(picture, x, y):
    if not isinstance(picture, Picture):
        print("getPixel(picture,x,y): First input is not a picture")
        raise ValueError
    if (x < Picture._PictureIndexOffset) or (x > getWidth(picture) - 1 + Picture._PictureIndexOffset):
        print("getPixel(picture,x,y): x (= {}) is less than {} or bigger than the width (= {})".format(x, Picture._PictureIndexOffset, getWidth(picture) - 1 + Picture._PictureIndexOffset))
        raise ValueError
    if (y < Picture._PictureIndexOffset) or (y > getHeight(picture) - 1 + Picture._PictureIndexOffset):
        print("getPixel(picture,x,y): y (= {}) is less than {} or bigger than the height (= {})".format(y, Picture._PictureIndexOffset, getHeight(picture) - 1 + Picture._PictureIndexOffset))
        raise ValueError

    return picture.getPixel(x - Picture._PictureIndexOffset, y - Picture._PictureIndexOffset)

# Added as a better name for getPixel


def getPixelAt(picture, x, y):
    return getPixel(picture, x, y)


def setRed(pixel, value):
    value = Pixel.correctLevel(value)
    if not isinstance(pixel, Pixel):
        print("setRed(pixel,value): Input is not a pixel")
        raise ValueError
    pixel.setRed(value)


def getRed(pixel):
    if not isinstance(pixel, Pixel):
        print("getRed(pixel): Input is not a pixel")
        raise ValueError
    return pixel.getRed()


def setBlue(pixel, value):
    value = Pixel.correctLevel(value)
    if not isinstance(pixel, Pixel):
        print("setBlue(pixel,value): Input is not a pixel")
        raise ValueError
    pixel.setBlue(value)


def getBlue(pixel):
    if not isinstance(pixel, Pixel):
        print("getBlue(pixel): Input is not a pixel")
        raise ValueError
    return pixel.getBlue()


def setGreen(pixel, value):
    value = Pixel.correctLevel(value)
    if not isinstance(pixel, Pixel):
        print("setGreen(pixel,value): Input is not a pixel")
        raise ValueError
    pixel.setGreen(value)


def getGreen(pixel):
    if not isinstance(pixel, Pixel):
        print("getGreen(pixel): Input is not a pixel")
        raise ValueError
    return pixel.getGreen()


def getColor(pixel):
    if not isinstance(pixel, Pixel):
        print("getColor(pixel): Input is not a pixel")
        raise ValueError
    return Color(pixel.getColor())


def setColor(pixel, color):
    if not isinstance(pixel, Pixel):
        print("setColor(pixel,color): First input is not a pixel")
        raise ValueError
    if not isinstance(color, Color):
        print("setColor(pixel,color): Second input is not a color")
        raise ValueError
    pixel.setColor(color)


def getX(pixel):
    if not isinstance(pixel, Pixel):
        print("getX(pixel): Input is not a pixel")
        raise ValueError
    return pixel.getX() + Picture._PictureIndexOffset


def getY(pixel):
    if not isinstance(pixel, Pixel):
        print("getY(pixel): Input is not a pixel")
        raise ValueError
    return pixel.getY() + Picture._PictureIndexOffset


def distance(c1, c2):
    if not isinstance(c1, Color):
        print("distance(c1,c2): First input is not a color")
        raise ValueError
    if not isinstance(c2, Color):
        print("distance(c1,c2): Second input is not a color")
        raise ValueError
    return c1.distance(c2)


def writePictureTo(picture, filename):
    global mediaFolder
    if not os.path.isabs(filename):
        filename = mediaFolder + filename
    if not isinstance(picture, Picture):
        print("writePictureTo(picture,filename): First input is not a picture")
        raise ValueError
    picture.writeOrFail(filename)
#   if not os.path.exists(filename):
#       print "writePictureTo(pict,filename): Path is not valid"
#       raise ValueError


# not to be confused with setColor, totally different, don't document/export
def _setColorTo(color, other):
    color.setRGB(other.getRed(), other.getGreen(), other.getBlue())
    return color

# def makeDarker(color):
    #"""This function has side effects on purpose, see p49 """
    # return _setColorTo(color, color.darker())


def makeDarker(color):
    if not isinstance(color, Color):
        print("makeDarker(color): Input is not a color")
        raise ValueError
    return Color(color.makeDarker())

# def makeLighter(color):
    #"""This function has side effects on purpose, see p49"""
    # return _setColorTo(color,color.brighter())


def makeLighter(color):
    if not isinstance(color, Color):
        print("makeLighter(color): Input is not a color")
        raise ValueError
    return Color(color.makeLighter())


def makeBrighter(color):  # This is the same as makeLighter(color)
    if not isinstance(color, Color):
        print("makeBrighter(color): Input is not a color")
        raise ValueError
    return Color(color.makeLighter())


def makeColor(red, green=None, blue=None):
    return Color(red, green, blue)


def setAllPixelsToAColor(picture, color):
    #"""This function sets the picture to one color"""
    if not isinstance(picture, Picture):
        print("setAllPixelsToAColor(picture,color): First input is not a picture")
        raise ValueError
    if not isinstance(color, Color):
        print("setAllPixelsToAColor(picture,color): Second input is not a color")
        raise ValueError
    picture.setAllPixelsToAColor(color)


# def copyInto(smallPicture, bigPicture, startX, startY):
#     # like copyInto(butterfly, jungle, 20,20)
#     if not isinstance(smallPicture, Picture):
#         print("copyInto(smallPicture, bigPicture, startX, startY): smallPicture must be a picture")
#         raise ValueError
#     if not isinstance(bigPicture,Picture):
#         print("copyInto(smallPicture, bigPicture, startX, startY): bigPicture must be a picture")
#         raise ValueError
#     if (startX < 0) or (startX > getWidth(bigPicture) - 1):
#         print("copyInto(smallPicture, bigPicture, startX, startY): startX must be within the bigPicture")
#         raise ValueError
#     if (startY < 0) or (startY > getHeight(bigPicture) - 1):
#         print("copyInto(smallPicture, bigPicture, startX, startY): startY must be within the bigPicture")
#         raise ValueError
#     if ((startX + getWidth(smallPicture) - 1) > (getWidth(bigPicture) - 1) or (startY + getHeight(smallPicture) - 1) > (getHeight(bigPicture) - 1)):
#         print("copyInto(smallPicture, bigPicture, startX, startY): smallPicture won't fit into bigPicture")
#         raise ValueError
# 
#     xOffset = startX - Picture._PictureIndexOffset
#     yOffset = startY - Picture._PictureIndexOffset
# 
#     for x in range(0, getWidth(smallPicture)):
#         for y in range(0, getHeight(smallPicture)):
#             bigPicture.setBasicPixel(
#                 x + xOffset, y + yOffset, smallPicture.getBasicPixel(x, y))
# 
#     return bigPicture

# Alyce Brady's version of copyInto, with additional error-checking on the upper-left corner
# Will copy as much of the original picture into the destination picture as will fit.
def copyInto(origPict, destPict, upperLeftX, upperLeftY):
 if not isinstance(origPict, Picture):
   print("copyInto(origPict, destPict, upperLeftX, upperLeftY): First parameter is not a picture")
   raise ValueError
 if not isinstance(destPict, Picture):
   print("copyInto(origPict, destPict, upperLeftX, upperLeftY): Second parameter is not a picture")
   raise ValueError
 if upperLeftX < 0 or upperLeftX > getWidth(destPict):
   print("copyInto(origPict, destPict, upperLeftX, upperLeftY): upperLeftX must be within the destPict")
   raise ValueError
 if upperLeftY < 0 or upperLeftY > getHeight(destPict):
   print("copyInto(origPict, destPict, upperLeftX, upperLeftY): upperLeftY must be within the destPict")
   raise ValueError
 return origPict.copyInto(destPict, upperLeftX-1, upperLeftY-1)


def duplicatePicture(picture):
    """returns a copy of the picture"""
    if not isinstance(picture, Picture):
        print("duplicatePicture(picture): Input is not a picture")
        raise ValueError
    return Picture(picture)

def cropPicture(picture, upperLeftX, upperLeftY, width, height):
 if not isinstance(picture, Picture):
   print("crop(picture, upperLeftX, upperLeftY, width, height): First parameter is not a picture")
   raise ValueError
 if upperLeftX < 1 or upperLeftX > getWidth(pic):
   print("crop(picture, upperLeftX, upperLeftY, width, height): upperLeftX must be within the picture")
   raise ValueError
 if upperLeftY < 1 or upperLeftY > getHeight(pic):
   print("crop(picture, upperLeftX, upperLeftY, width, height): upperLeftY must be within the picture")
   raise ValueError
 return picture.crop(upperLeftX-1, upperLeftY-1, width, height)

##
# Input and Output interfaces
#
# Note: These calls must be done in a threadsafe manner since the JESThread will be
# executing them rather than the GUI's event dispatch thread.
# See {Simple,Stoppable}{Input,Output}.java for the threadsafe execution.
##


# def requestNumber(message):
#     return StoppableInput.getNumber(message)


# def requestInteger(message):
#     return StoppableInput.getIntNumber(message)


# def requestIntegerInRange(message, min, max):
#     if min >= max:
#         print("requestIntegerInRange(message, min, max): min >= max not allowed")
#         raise ValueError

#     return StoppableInput.getIntNumber(message, min, max)


# def requestString(message):
#     s = StoppableInput.getString(message)
#     if s is None:
#         return None
#     else:
#         return str(s)


# def input(message=None):
#     from jes.gui.commandwindow.prompt import promptService
#     return eval(promptService.requestInput(message))


# def raw_input(message=None):
#     from jes.gui.commandwindow.prompt import promptService
#     return promptService.requestInput(message)


# def showWarning(message):
#     return StoppableOutput.showWarning(message)


# def showInformation(message):
#     return StoppableOutput.showInformation(message)


# def showError(message):
#     return StoppableOutput.showError(message)


##
# Java Music Interface
##
# def playNote(note, duration, intensity=64):
#     JavaMusic.playNote(note, duration, intensity)


##
# General user tools
#

def pickAFile():
    # Note: this needs to be done in a threadsafe manner, see FileChooser
    # for details how this is accomplished.
    return str(FileChooser.pickAFile())


def pickAFolder():
    # Note: this needs to be done in a threadsafe manner, see FileChooser
    # for details how this is accomplished.
    dir = FileChooser.pickADirectory()
    if (dir != None):
        return str(dir + os.sep)
    return None


def quit():
    sys.exit(0)

##
# MediaTools interface
#
# TODO modify viewer.changeToBaseOne


def openPictureTool(picture):
    #import PictureExplorer
    #thecopy = duplicatePicture(picture)
    #viewer = PictureExplorer(thecopy)

#    viewer.changeToBaseOne();
    #viewer.setTitle(getShortPath(picture.getFileName()))
    picture.pictureTool()


def openFrameSequencerTool(movie):
    FrameSequencerTool(movie)


def openSoundTool(sound):
    import SoundExplorer
    thecopy = Sound(sound)
    viewer = SoundExplorer(thecopy, 0)
    try:
        viewer.setTitle(getShortPath(sound.getFileName()))
    except:
        viewer.setTitle("No File Name")


def explore(someMedia):
    if isinstance(someMedia, Picture):
        openPictureTool(someMedia)
    elif isinstance(someMedia, Sound):
        openSoundTool(someMedia)
    elif isinstance(someMedia, Movie):
        openFrameSequencerTool(someMedia)
    else:
        print("explore(someMedia): Input is not a Picture, Sound, or Movie")
        raise ValueError

# let's try the turtles...
# import Turtle
# import World


def turn(turtle, degrees=90):
    if not isinstance(turtle, Turtle):
        print("turn(turtle[, degrees]): Input is not a turtle")
        raise ValueError
    else:
        turtle.turn(degrees)


def turnRight(turtle):
    if not isinstance(turtle, Turtle):
        print("turnRight(turtle): Input is not a turtle")
        raise ValueError
    else:
        turtle.turnRight()


def turnToFace(turtle, x, y=None):
    if y == None:
        if not (isinstance(turtle, Turtle) and isinstance(x, Turtle)):
            print("turnToFace(turtle, turtle): First input is not a turtle")
            raise ValueError
        else:
            turtle.turnToFace(x)
    else:
        if not isinstance(turtle, Turtle):
            print("turnToFace(turtle, x, y): Input is not a turtle")
            raise ValueError
        else:
            turtle.turnToFace(x, y)


def turnLeft(turtle):
    if not isinstance(turtle, Turtle):
        print("turnLeft(turtle): Input is not a turtle")
        raise ValueError
    else:
        turtle.turnLeft()


def forward(turtle, pixels=100):
    if not isinstance(turtle, Turtle):
        print("forward(turtle[, pixels]): Input is not a turtle")
        raise ValueError
    else:
        turtle.forward(pixels)


def backward(turtle, pixels=100):
    if not isinstance(turtle, Turtle):
        print("backward(turtle[, pixels]): Input is not a turtle")
        raise ValueError
    if (None == pixels):
        turtle.backward()
    else:
        turtle.backward(pixels)


def moveTo(turtle, x, y):
    if not isinstance(turtle, Turtle):
        print("moveTo(turtle, x, y): Input is not a turtle")
        raise ValueError
    turtle.moveTo(x, y)


def makeTurtle(world):
    if not (isinstance(world, World) or isinstance(world, Picture)):
        print("makeTurtle(world): Input is not a world or picture")
        raise ValueError
    turtle = Turtle(world)
    return turtle


def penUp(turtle):
    if not isinstance(turtle, Turtle):
        print("penUp(turtle): Input is not a turtle")
        raise ValueError
    turtle.penUp()


def penDown(turtle):
    if not isinstance(turtle, Turtle):
        print("penDown(turtle): Input is not a turtle")
        raise ValueError
    turtle.penDown()


def drop(turtle, picture):
    if not isinstance(turtle, Turtle):
        print("drop(turtle, picture): First input is not a turtle")
        raise ValueError
    if not isinstance(picture, Picture):
        print("drop(turtle, picture): Second input is not a picture")
        raise ValueError
    turtle.drop(picture)


def getXPos(turtle):
    if not isinstance(turtle, Turtle):
        print("getXPos(turtle): Input is not a turtle")
        raise ValueError
    return turtle.getXPos()


def getYPos(turtle):
    if not isinstance(turtle, Turtle):
        print("getYPos(turtle): Input is not a turtle")
        raise ValueError
    return turtle.getYPos()


def getHeading(turtle):
    if not isinstance(turtle, Turtle):
        print("getHeading(turtle): Input is not a turtle")
        raise ValueError
    return turtle.getHeading()

# add these things: turnToFace(turtle, another turtle)
## getHeading, getXPos, getYPos

# world methods


def makeWorld(width=None, height=None):
    if(width and height):
        w = World(width, height)
    else:
        w = World()
    return w


def getTurtleList(world):
    if not isinstance(world, World):
        print("getTurtleList(world): Input is not a world")
        raise ValueError
    return world.getTurtleList()

# end of stuff imported for worlds and turtles

# used in the book


def printNow(text):
    print(text)


class Movie(object):
    def __init__(self):  # frames are filenames
        self.frames = []
        self.dir = None

    def addFrame(self, frame):
        self.frames.append(frame)
        self.dir = None

    def __len__(self):
        return len(self.frames)

    def __str__(self):
        return "Movie, frames: " + str(len(self))

    def __repr__(self):
        return "Movie, frames: " + str(len(self))

    def __getitem__(self, item):
        return self.frames[item]

    def writeFramesToDirectory(self, directory):
        import FrameSequencer
        fs = FrameSequencer(directory)
        # for frameindex in range(0, self.listModel.size()):
        # fs.addFrame(Picture(self.listModel.get(frameindex)))
        # fs.play(self.fps)
        for frameindex in range(0, len(self.frames)):
            fs.addFrame(Picture(self.frames[frameindex]))
        self.dir = directory

    def play(self):
        import java.util.ArrayList as ArrayList
        list = ArrayList()
        for f in self.frames:
            list.add(makePicture(f))
        MoviePlayer(list).playMovie()

    # def writeQuicktime(self, destPath, framesPerSec=16):
    #     global mediaFolder
    #     if not os.path.isabs(destPath):
    #         destPath = mediaFolder + destPath
    #     destPath = "file://" + destPath
    #     if framesPerSec <= 0:
    #         print("writeQuicktime(path[, framesPerSec]): Frame Rate must be a positive number")
    #         raise ValueError
    #     if self.frames == []:  # Is movie empty?
    #         print("writeQuicktime(path[, framesPerSec]): Movie has no frames. Cannot write empty Movie")
    #         raise ValueError
    #     # Is movie only 1 frame but never written out
    #     elif self.dir == None and len(self.frames) == 1:
    #         frame = self.frames[0]
    #         self.dir = frame[:(frame.rfind(os.sep))]
    #     # Are movie frames all in the same directory?
    #     elif self.dir == None and len(self.frames) > 1:
    #         sameDir = 1
    #         frame = self.frames[0]
    #         frame = frame.replace('/', os.sep)
    #         # Parse directory of first frame
    #         framesDir = frame[:(frame.rfind(os.sep))]
    #         thisDir = framesDir
    #         frameNum = 1
    #         while(sameDir and frameNum < len(self.frames)):
    #             frame = self.frames[frameNum]
    #             # Eliminate possibility of / vs. \ causing problems
    #             frame = frame.replace('/', os.sep)
    #             thisDir = frame[:(frame.rfind(os.sep))]
    #             frameNum = frameNum + 1
    #             if(framesDir <> thisDir):
    #                 sameDir = 0
    #         if(sameDir):  # Loop ended because we ran out of frames
    #             self.dir = framesDir
    #         else:  # Loop ended because sameDir became false
    #             print("writeQuicktime(path[, framesPerSec]): Your frames are in different directories. Call writeFramesToDirectory() first, then try again.")
    #             raise ValueError
    #     writer = MovieWriter(self.dir, framesPerSec, destPath)
    #     writer.writeQuicktime()

    # def writeAVI(self, destPath, framesPerSec=16):
    #     global mediaFolder
    #     if not os.path.isabs(destPath):
    #         destPath = mediaFolder + destPath
    #     destPath = "file://" + destPath
    #     if framesPerSec <= 0:
    #         print("writeAVI(path[, framesPerSec]): Frame Rate must be a positive number")
    #         raise ValueError
    #     if self.frames == []:  # Is movie empty?
    #         print("writeAVI(path[, framesPerSec]): Movie has no frames. Cannot write empty Movie")
    #         raise ValueError
    #     # Is movie only 1 frame but never written out
    #     elif self.dir == None and len(self.frames) == 1:
    #         frame = self.frames[0]
    #         self.dir = frame[:(frame.rfind(os.sep))]
    #     # Are movie frames all in the same directory?
    #     elif self.dir == None and len(self.frames) > 1:
    #         sameDir = 1
    #         frame = self.frames[0]
    #         frame = frame.replace('/', os.sep)
    #         # Parse directory of first frame
    #         framesDir = frame[:(frame.rfind(os.sep))]
    #         thisDir = framesDir
    #         frameNum = 1
    #         while(sameDir and frameNum < len(self.frames)):
    #             frame = self.frames[frameNum]
    #             frame = frame.replace('/', os.sep)
    #             thisDir = frame[:(frame.rfind(os.sep))]
    #             frameNum = frameNum + 1
    #             if(framesDir <> thisDir):
    #                 sameDir = 0
    #         if(sameDir):  # Loop ended because we ran out of frames
    #             self.dir = framesDir
    #         else:  # Loop ended because sameDir became false
    #             print("writeAVI(path[, framesPerSec]): Your frames are in different directories. Call writeFramesToDirectory() first, then try again.")
    #             raise ValueError
    #     writer = MovieWriter(self.dir, framesPerSec, destPath)
    #     writer.writeAVI()


def playMovie(movie):
    if isinstance(movie, Movie):
        movie.play()
    else:
        print("playMovie( movie ): Input is not a Movie")
        raise ValueError


def writeQuicktime(movie, destPath, framesPerSec=16):
    if not (isinstance(movie, Movie)):
        print("writeQuicktime(movie, path[, framesPerSec]): First input is not a Movie")
        raise ValueError
    if framesPerSec <= 0:
        print("writeQuicktime(movie, path[, framesPerSec]): Frame rate must be a positive number")
        raise ValueError
    movie.writeQuicktime(destPath, framesPerSec)


def writeAVI(movie, destPath, framesPerSec=16):
    if not (isinstance(movie, Movie)):
        print("writeAVI(movie, path[, framesPerSec]): First input is not a Movie")
        raise ValueError
    if framesPerSec <= 0:
        print("writeAVI(movie, path[, framesPerSec]): Frame rate must be a positive number")
        raise ValueError
    movie.writeAVI(destPath, framesPerSec)


def makeMovie():
    return Movie()


def makeMovieFromInitialFile(filename):
    import re
    movie = Movie()

    #filename = filename.replace(os.altsep, os.sep)
    # Hack fix because os.altsep is not defined for Windows as of Python 2.2
    filename = filename.replace('/', os.sep)
    sep_location = filename.rfind(os.sep)
    if(-1 == sep_location):
        filename = mediaFolder + filename

    movie.directory = filename[:(filename.rfind(os.sep))]
    movie.init_file = filename[(filename.rfind(os.sep)) + 1:]
    regex = re.compile('[0-9]+')
    file_regex = regex.sub('.*', movie.init_file)

    for item in sorted(os.listdir(movie.directory)):
        if re.match(file_regex, item):
            movie.addFrame(movie.directory + os.sep + item)

    return movie


def addFrameToMovie(a, b):
    frame = None
    movie = None
    if a.__class__ == Movie:
        movie = a
        frame = b
    else:
        movie = b
        frame = a

    if not (isinstance(movie, Movie) and isinstance(frame, String)):
       # if movie.__class__ != Movie or frame.__class__ != String:
        print("addFrameToMovie(frame, movie): frame is not a string or movie is not a Movie object")
        raise ValueError

    movie.addFrame(frame)


def writeFramesToDirectory(movie, directory=None):
    if not isinstance(movie, Movie):
        print("writeFramesToDirectory(movie[, directory]): movie is not a Movie object")
        raise ValueError

    if directory == None:
        directory = user.home

    movie.writeFramesToDirectory(directory)

# def playMovie(movie):
#    if not isinstance(movie, Movie):
#        print "playMovie(movie): movie is not a Movie object."
#        raise ValueError
#    movie.play()



#!/usr/bin/env python3

"""
pictureTool.py - script program to implement the "explore" functionality
                 in JES for the JES4py package

Written: 2020-07-24 Gahngnin Kim <gahngnin.kim@gordon.edu>
Modified: 2020-08-08 Jonathan Senning <jonathan.senning@gordon.edu>
- Completed work for crosshair cursor to work on Mac, Windows, and Linux (GTK)

The "explore()" function in JES will open a new window and display the image
imported from the given file path. The window provides an interactive picture
tool for the user. It allows users to zoom in and out of the image and pick
a pixel to examine its RGB values with the color preview. When a pixel is
selected from the image, a crosshair will appear in that selected position.
Unlike, "show()" function, it cannot be repainted with "repaint()" method.

The JES4py's implementation of "explore()" function provides a nearly identical
experience with an improvement compared to JES's. Display and image size
detection code is added to ensure that the window size doesn't go beyond the
display resolution even if the image is larger. The image panel will set
scrollbars when a high-resolution image is imported will set the scrollbar to
fit the image in the window. The program can also work standalone.

This program was first developed during the Gordon College Computer Science
Summer 2020 Practicum as a part of the JES4py project, under the guidance of
Dr. Jonathan Senning.

Summer 2020 JES4py Team: Dr. Jonathan Senning
                         Nick Noormand
                         Gahngnin Kim

"""

import os
import sys
import wx
import wx.lib.scrolledpanel
# import wx.lib.inspection

class Cursor:
    width  = 7
    height = 7
    centerX = int((width - 1)/2)
    centerY = int((height - 1)/2)
    cursorBitmap = None
    savedCursorPosition = None
    cursorBackupBitmap = None

    def __init__(self, width=7, height=7):
        self.width = width
        self.height = height
        self.centerX = int((self.width - 1)/2)
        self.centerY = int((self.height - 1)/2)

        if wx.Platform != "__WXMAC__":
            # WX on some macs does not support monochrome bitmaps which
            # are necessary for bitmap masks.  Thus we don't need to do
            # anything here
            dc = wx.MemoryDC()

            # Create the mask bitmap
            cursorMask = wx.Bitmap(self.width, self.height, depth=1)
            dc.SelectObject(cursorMask)
            dc.SetPen(wx.Pen(wx.Colour(255, 255, 255), width=3))
            dc.DrawLine(0, self.centerY, self.width-1, self.centerY)
            dc.DrawLine(self.centerX, 0, self.centerX, self.height-1)

            # Create the cursor bitmap
            self.cursorBitmap = wx.Bitmap(self.width, self.height)
            dc.SelectObject(self.cursorBitmap)
            dc.SetPen(wx.Pen(wx.Colour(255, 255, 0)))
            dc.DrawLine(0, self.centerY, self.width-1, self.centerY)
            dc.DrawLine(self.centerX, 0, self.centerX, self.height-1)
            self.cursorBitmap.SetMask(wx.Mask(cursorMask))

            # done with dc
            del dc

    def drawCursor(self, canvas, x, y):
        # Get scaled image size
        W, H = canvas.image.GetSize()
        W, H = int(W * canvas.zoomFactor), int(H * canvas.zoomFactor)

        # Compute size and position of cursor and save corresponding portion
        # of the image 
        w, h = self.width, self.height
        dx, dy = self.centerX, self.centerY
        x0 = max(0, x-dx)
        y0 = max(0, y-dy)
        w0 = dx + min(dx+1, W-x)
        h0 = dy + min(dy+1, H-y)
        cursorRect = wx.Rect(x0, y0, w0, h0)
        cursorPosition = x-dx, y-dy
        self.savedCursorPosition = x0, y0
        self.cursorBackupBitmap = canvas.bmp.GetSubBitmap(cursorRect)
        # print(f"Cursor size: {self.cursorSize}, Cursor Rect: {cursorRect}")

        # Draw the cursor - use bitmap on Windows and GTK, DC Draw on Mac
        if wx.Platform == "__WXMSW__":
            dc = wx.ClientDC(canvas.imageCtrl)
        else:
            dc = wx.ClientDC(canvas.imagePanel)
            canvas.imagePanel.DoPrepareDC(dc)
        dc.SetClippingRegion(0, 0, W, H)
        if self.cursorBitmap is None:
            # We need to manually draw the cursor
            dx, dy = self.centerX-1, self.centerY-1
            dc.SetPen(wx.Pen(wx.Colour(0, 0, 0), width=3))
            dc.DrawLine(x-dx, y, x+dx, y)
            dc.DrawLine(x, y-dy, x, y+dy)
            dc.SetPen(wx.Pen(wx.Colour(255, 255, 0), width=1))
            dc.DrawLine(x-dx, y, x+dx, y)
            dc.DrawLine(x, y-dy, x, y+dy)
        else:
            # Display cursor bitmap
            dc.DrawBitmap(self.cursorBitmap, cursorPosition, True)
        del dc

    def undrawPreviousCursor(self, canvas):
        """Restore bitmap (if any) saved from previous cursor event
        """
        if self.cursorBackupBitmap is not None:
            if wx.Platform == "__WXMSW__":
                dc = wx.ClientDC(canvas.imageCtrl)
            else:
                dc = wx.ClientDC(canvas.imagePanel)
                canvas.imagePanel.DoPrepareDC(dc)
            dc.DrawBitmap(self.cursorBackupBitmap, self.savedCursorPosition, False)
            del dc

    def drawCrosshairs(self, canvas):
        """Draw image with crosshairs to indicate selected position
        """
        # Get coordinates of current cursor location
        x = int(int(canvas.pixelTxtX.GetValue()) * canvas.zoomFactor)
        y = int(int(canvas.pixelTxtY.GetValue()) * canvas.zoomFactor)

        # Restore bitmap (if any) saved from previous cursor event
        # and draw new cursor
        self.undrawPreviousCursor(canvas)
        self.drawCursor(canvas, x, y)

    def clearBackupBitmap(self):
        """Forget saved bitmap buffer
        """
        if self.cursorBackupBitmap is not None:
            self.cursorBackupBitmap = None

class MainWindow(wx.Frame):

    MinWindowWidth = 350
    MinWindowHeight = 0
    ColorPanelHeight = 70
    PaintChipSize = 24
    zoomLevels = [25, 50, 75, 100, 150, 200, 500]
    zoomFactor = float(zoomLevels[3]) / 100.0
    cursorSize = (7, 7)
    cursorBitmap = None
    x = 0
    y = 0

    def __init__(self, filename, parent, title):
        # Load image and get image size
        self.image = wx.Image(filename, wx.BITMAP_TYPE_ANY)
        self.bmp = wx.Bitmap(self.image)
        super(MainWindow, self).__init__(parent=parent, title=title, style=wx.DEFAULT_FRAME_STYLE)

        self.InitUI()
        self.Center()
        self.clipOnBoundary()
        # wx.lib.inspection.InspectionTool().Show() # Inspection tool for debugging

    def InitUI(self):
        # Get display size and reduce width and height by small percentage
        wView, hView = wx.DisplaySize()
        wView, hView = int(wView * 0.95), int(hView * 0.85)

        # Get image dimensions and adjust window size to hold as much of the image
        # as possible but stil be within display
        w, h = self.image.GetSize()
        w = min(max(self.MinWindowWidth, w), wView)
        h = min(max(self.MinWindowHeight, h + self.ColorPanelHeight), hView)

        # Setup the zoom menu
        self.setupZoomMenu()

        # Create main sizer
        self.mainSizer = wx.BoxSizer(wx.VERTICAL)

        # Setup the color information panel
        self.setupColorInfoDisplay()
        self.mainSizer.Add(self.colorInfoPanel, 0, wx.EXPAND|wx.ALL, 0)

        # Set up the image display panel
        self.setupImageDisplay()
        self.mainSizer.Add(self.imagePanel, 0, wx.EXPAND|wx.ALL, 0)
        self.SetSizer(self.mainSizer)
        self.Fit()

        # Set window sizes
        self.SetSize((w, h))
        self.SetClientSize((w,h))

    def setupZoomMenu(self):
        # Set up zoom menu
        zoomMenu = wx.Menu()
        for z in self.zoomLevels:
            item = "{}%".format(z)
            helpString = "Zoom by {}%".format(z)
            zoomID = zoomMenu.Append(wx.ID_ANY, item, helpString)
            self.Bind(wx.EVT_MENU, self.onZoom, zoomID)

        # Create the menubar
        menuBar = wx.MenuBar()
        menuBar.Append(zoomMenu, "&Zoom") # Adds the "zoomMenu" to the MenuBar
        self.SetMenuBar(menuBar) # Adds the MenuBar to the Frame content.

    def setupColorInfoDisplay(self):
        self.colorInfoPanel = wx.Panel(parent=self, size=(-1, self.ColorPanelHeight))

        # Create main sizer for this color info panel
        mainSizer = wx.BoxSizer(wx.VERTICAL)

        # ---- First row ----

        # Create sizer to hold components
        sizer1 = wx.BoxSizer(wx.HORIZONTAL)

        # X and Y labels
        lblX = wx.StaticText(self.colorInfoPanel, 0, style=wx.ALIGN_CENTER)
        lblY = wx.StaticText(self.colorInfoPanel, 0, style=wx.ALIGN_CENTER)
        lblX.SetLabel("X: ")
        lblY.SetLabel("Y: ")
    
        # Navigation buttons (Enable after getting the colorpicker/eyedropper functional)
        # Icons made by Freepik from www.flaticon.com; Modified by Gahngnin Kim
        img_R = os.path.join(sys.path[0], 'images', 'Right.png')
        img_L = os.path.join(sys.path[0], 'images', 'Left.png')
        bmp_R = wx.Bitmap(img_R, wx.BITMAP_TYPE_ANY)
        bmp_L = wx.Bitmap(img_L, wx.BITMAP_TYPE_ANY)
        bmp_size = (bmp_L.GetWidth()+5,bmp_L.GetHeight()+5)
        buttonX_L = wx.BitmapButton(self.colorInfoPanel, wx.ID_ANY, bitmap=bmp_L, size=bmp_size)
        buttonX_L.myname = "XL"
        buttonX_R = wx.BitmapButton(self.colorInfoPanel, wx.ID_ANY, bitmap=bmp_R, size=bmp_size)
        buttonX_R.myname = "XR"
        buttonY_L = wx.BitmapButton(self.colorInfoPanel, wx.ID_ANY, bitmap=bmp_L, size=bmp_size)
        buttonY_L.myname = "YL"
        buttonY_R = wx.BitmapButton(self.colorInfoPanel, wx.ID_ANY, bitmap=bmp_R, size=bmp_size)
        buttonY_R.myname = "YR"
        buttonX_L.Bind(wx.EVT_BUTTON, self.ImageCtrl_OnNavBtn)
        buttonX_R.Bind(wx.EVT_BUTTON, self.ImageCtrl_OnNavBtn)
        buttonY_L.Bind(wx.EVT_BUTTON, self.ImageCtrl_OnNavBtn)
        buttonY_R.Bind(wx.EVT_BUTTON, self.ImageCtrl_OnNavBtn)

        # Textboxes to display X and Y coordinates on click
        self.pixelTxtX = wx.TextCtrl(self.colorInfoPanel, wx.ALIGN_CENTER, \
            style=wx.TE_PROCESS_ENTER, size=(50, -1))
        self.pixelTxtY = wx.TextCtrl(self.colorInfoPanel, wx.ALIGN_CENTER, \
            style=wx.TE_PROCESS_ENTER, size=(50, -1))
        self.pixelTxtX.Bind(wx.EVT_TEXT_ENTER, self.ImageCtrl_OnEnter)
        self.pixelTxtY.Bind(wx.EVT_TEXT_ENTER, self.ImageCtrl_OnEnter)

        # Add first row components to sizer
        sizer1.Add(lblX, 0, flag=wx.CENTER, border=0) # X coordinate display
        sizer1.Add(buttonX_L, 0, border=0)
        sizer1.Add(self.pixelTxtX, 0, flag=wx.CENTER, border=5) # Text Control Box
        sizer1.Add(buttonX_R, 0, border=0)
        sizer1.Add((5, -1)) # Horizonal spacer
        sizer1.Add(lblY, 0, flag=wx.CENTER, border=0) # Y coordinate display
        sizer1.Add(buttonY_L, 0, border=0)
        sizer1.Add(self.pixelTxtY, 0, flag=wx.CENTER, border=5) # Text Control Box
        sizer1.Add(buttonY_R, 0, border=0)
        
        # Add first row sizer to the main sizer
        mainSizer.Add(sizer1, 0, flag=wx.LEFT|wx.RIGHT|
                    wx.TOP|wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, border=1)
        mainSizer.Add((-1, 5)) # Vertical spacer

        # ---- Second row ----

        # Create sizer to hold components
        sizer2 = wx.BoxSizer(wx.HORIZONTAL)

        # Static text displays RGB values of the given coordinates
        # Initialized with dummie values
        self.rgbValue = wx.StaticText(self.colorInfoPanel, label=u'')#, style=wx.ALIGN_CENTER)
        font = self.rgbValue.GetFont()
        font.PointSize += 2
        self.rgbValue.Font = font

        # initialize an empty bitmap
        bmp = wx.Bitmap(self.PaintChipSize, self.PaintChipSize)
        self.colorPreview = wx.StaticBitmap(parent=self.colorInfoPanel, bitmap=bmp)

        # Add items to the second sizer (sizer2)
        sizer2.Add(self.rgbValue, 0, flag=wx.ALIGN_CENTER, border=5)
        sizer2.Add((10, -1), 0) # Horizontal spacer

        # Small image that shows the color at the selected pixel
        sizer2.Add(self.colorPreview, 0, flag=wx.ALIGN_CENTER, border=5) 

        # Add sizer2 to the main sizer
        mainSizer.Add(sizer2, 0, flag=wx.ALIGN_CENTER_HORIZONTAL|wx.ALL, border=1)
        mainSizer.Add((-1, 5)) # Vertical spacer

        self.colorInfoPanel.SetSizer(mainSizer)

    def setupImageDisplay(self):
        """Set up image display panel
        """
        # Get image size and make scrolled panel large enough to hold image
        # even with maximum zoom
        w, h = self.image.GetSize()
        maxZoomLevel = int(int(self.zoomLevels[-1]) / 100)
        maxSize = (maxZoomLevel * w, maxZoomLevel * h)
    
        # Create a scrolled panel to hold the image
        self.imagePanel = wx.lib.scrolledpanel.ScrolledPanel(parent=self, size=maxSize, style=wx.NO_BORDER)

        # Store the image and setup even handlers for mouse clicks and motion
        self.imageCtrl = wx.StaticBitmap(parent=self.imagePanel, bitmap=self.bmp)
        if wx.Platform == "__WXMSW__" or wx.Platform == "__WXGTK__":
            self.imageCtrl.Bind(wx.EVT_LEFT_DOWN, self.ImageCtrl_OnMouseClick)
            self.imageCtrl.Bind(wx.EVT_MOTION, self.ImageCtrl_OnMouseClick)
        elif wx.Platform == "__WXMAC__":
            self.imagePanel.Bind(wx.EVT_LEFT_DOWN, self.ImageCtrl_OnMouseClick)
            self.imagePanel.Bind(wx.EVT_MOTION, self.ImageCtrl_OnMouseClick) 

        # Add image control to image panel
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        mainSizer.Add(self.imageCtrl, 0, wx.ALIGN_LEFT|wx.ALIGN_TOP|wx.ALL, 0)
        self.imagePanel.SetSizer(mainSizer)
        self.mainSizer.Fit(self.imagePanel)
        self.imagePanel.SetupScrolling(scrollToTop=True)

        # Create cursor
        self.crosshair = Cursor()

    def clipOnBoundary(self):
        """Clips x and y to be valid pixel coordinates

        Ensures that the current values of x and y are valid pixel
        coordinates for the image

        Modifies
        --------
        self.x, self.y : int
            the pixel coordinates
        self.pixelTxtX, self.pixelTxtY : wx.TextCtrl
            the textboxes displaying the pixel coordinates
        """
        width, height = self.image.GetSize()
        if self.x < 0:
            self.x = 0
        elif self.x >= width:
            self.x = width - 1
        if self.y < 0:
            self.y = 0
        elif self.y >= height:
            self.y = height - 1
        self.pixelTxtX.SetValue(str(self.x))
        self.pixelTxtY.SetValue(str(self.y))
        self.updateColorInfo()

    def updateColorInfo(self):
        """Updates the color patch in the color information display

        The color patch is a small square image with a black outline and
        interior color matching that corresponding to the most-recently
        selected pixel in the image.
        """
        r = self.image.GetRed(self.x, self.y)
        g = self.image.GetGreen(self.x, self.y)
        b = self.image.GetBlue(self.x, self.y)
        self.rgbValue.SetLabel(label=u'R: {} G: {} B: {}  Color at location:'.format(r, g, b))

        # Sets the color of the square image on mouse click
        bmp = wx.Bitmap(self.PaintChipSize, self.PaintChipSize)
        dc = wx.MemoryDC()
        dc.SelectObject(bmp)
        dc.SetPen(wx.Pen(wx.Colour(0,0,0))) # black for outline
        dc.SetBrush(wx.Brush(wx.Colour(r,g,b))) # current image pixel color
        dc.DrawRectangle(0, 0, self.PaintChipSize, self.PaintChipSize)
        del dc
        self.colorPreview.SetBitmap(bmp) # Replace the bitmap with the new one
        self.colorInfoPanel.Layout()

    def updateView(self):
        """Scale image according to the zoom factor and (re)display it
        """
        w, h = self.image.GetSize()
        w, h = int(w * self.zoomFactor), int(h * self.zoomFactor)
        image = self.image.Scale(w, h)
        self.bmp = wx.Bitmap(image)
        self.imageCtrl.SetBitmap(self.bmp)

        # event = wx.PyCommandEvent(wx.EVT_TEXT_ENTER.typeId, self.GetId())
        # wx.PostEvent(self.GetEventHandler(), event)

        # Forget any saved bitmap buffer
        self.crosshair.clearBackupBitmap()
        
    def drawCrosshairs(self):
        """Draw image with crosshairs to indicate selected position
        """
        self.crosshair.drawCrosshairs(self)

# ===========================================================================
# Event handlers
# ===========================================================================

    def onFocus(self, event):
        self.imagePanel.SetFocus()

    def onZoom(self, event):
        """Sets desired zoom level and updates image display
        """
        id_selected = event.GetId() # Gets the event id of the selected menu item
        obj = event.GetEventObject() # Gets the event object
        menuItem = obj.GetLabelText(id_selected) # Gets the label text of the menu item
        self.zoomFactor = float(menuItem.replace('%','')) / 100.0

        # Reset position of panel
        self.imagePanel.Scroll(0, 0) # "non-scrolled" position

        self.PostSizeEvent()
        self.clipOnBoundary()
        self.updateView()

    def ImageCtrl_OnNavBtn(self, event):
        """Increment or decrement x or y pixel coordinate
        """
        selectedBtn = event.GetEventObject().myname
        if selectedBtn == "XL":
            self.x = int(self.pixelTxtX.GetValue()) - 1
        elif selectedBtn == "XR":
            self.x = int(self.pixelTxtX.GetValue()) + 1
        elif selectedBtn == "YL":
            self.y = int(self.pixelTxtY.GetValue()) - 1
        elif selectedBtn == "YR":
            self.y = int(self.pixelTxtY.GetValue()) + 1
        self.clipOnBoundary()
        self.drawCrosshairs()

    def ImageCtrl_OnEnter(self, event):
        """Adjusts x and y pixel values to match displayed values
        """
        self.x = int(self.pixelTxtX.GetValue())
        self.y = int(self.pixelTxtY.GetValue())
        self.clipOnBoundary()
        self.drawCrosshairs()

    def ImageCtrl_OnMouseClick(self, event):
        """Update x and y pixel coordinates from pointer location
        """
        if event.LeftIsDown():
            event.Skip()
            if wx.Platform == "__WXMSW__":
                cursorPosition = event.GetPosition()
            elif wx.Platform == "__WXGTK__" or wx.Platform == "__WXMAC__":
                dc = wx.ClientDC(self)
                self.imagePanel.DoPrepareDC(dc)
                cursorPosition = event.GetLogicalPosition(dc)
                del dc

            self.x = int(cursorPosition.x / self.zoomFactor)
            self.y = int(cursorPosition.y / self.zoomFactor)
            # print(f"\x1b[1mcursorPosition = {self.cursorPosition}\x1b[0m")

            self.clipOnBoundary()
            self.drawCrosshairs()

# ===========================================================================
# Main program
# ===========================================================================

def main(argv):

    usage = "usage: {} file [title]".format(argv[0])
    # Get image file name and optional image title from command line
    if len(argv) == 2:
        filename = title = argv[1]
    elif len(argv) == 3:
        filename = argv[1]
        title = argv[2]
    else:
        print(usage)
        exit(1)

    if not os.path.isfile(filename):
        print("{} does not exist or is not a file".format(filename))
        print(usage)
        exit(1)

    app = wx.App(False)
    frame = MainWindow(filename=filename, parent=None, title=title)
    frame.Show()
    app.MainLoop()

if __name__ == '__main__':
    main(sys.argv)



from jes4py.Sample import Sample

class Samples:
    def __init__(self, sound):
        self.sound = sound
        self.samples = []
        for i in range(sound.getLength()):
            self.samples.append(Sample(sound, i))

    def __str__(self):
        """Obtains a string representation of this array of Samples

        Returns
        -------
        str
            string representation of this array of Samples
        """
        return "Samples, length {}".format(self.sound.getLength())

    def __retr__(self):
        return self.__str__()

    @classmethod
    def getSamples(cls, sound):
        """Method to get the array of samples from a sound

        Parameters
        ----------
        sound : Sound
            the sound

        Returns
        -------
        list of Sample
            the array of samples
        """
        samples = []
        for i in range(sound.getLength()):
            samples.append(Sample(sound, i))
        return samples

    def getSample(self, index):
        """Method to get a specific Sample

        Parameters
        ----------
        index : int
            the index to get the sample from

        Returns
        -------
        Sample
            the sample at the given index
        """
        return self.samples[index]

    def setSample(self, index, value):
        """Method to set the value of a specific Sample

        Parameters
        ----------
        index : int
           the index to get the sample
        value : int or float
            the value to set it to
        """
        self.samples[index].setValue(value)

    def getSound(self):
        """Method to get these Samples' sound object

        Returns
        -------
        Sound
            the sound object
        """
        return self.sound



#import os, sys
import simpleaudio as sa
# import numpy as np
import wave
from jes4py import Config
from jes4py.SoundSample import SoundSample
#import FileChooser

class Sound:
    MAX_NEG = -32768
    MAX_POS = 32767
    SAMPLE_RATE = 22050
    NUM_BITS_PER_SAMPLE = 16
    _SoundIndexOffset = 0

    def __init__(self, sound, sampleRate=22050):
        """Construct new sound object
        
        If first passed parameter is the name of a WAV file, then read
        the file.

        If first passed parameter is an int representing a number of
        frames, then construct Sound object of the specified length.
        This sound will simply consist of an empty byte array and an
        AudioFileFormat with the following values:
            AudioFileFormat.Type.WAVE</code>
            22.05K sampling rate
            16 bit sample
            1 channel
            signed PCM encoding
            small-endian byte order
        Note that no new sound file is created, we only represent the
        sound with a buffer and the AudioFileFormat.  If a file is
        desired, then the method writeToFile(filename) must be called
        on this newly created sound.
    
        Parameters
        ----------
        sound : str
            the filename of containing a sound in WAV format
        sound : int
            the number of samples in the sound
        sound : Sound
            a preexisting Sound object
        sampleRate : int
            the frame rate for the sound
        """
        if isinstance(sound, str):
            self.filename = sound
            waveRead = wave.open(self.filename, 'rb')
            self.numFrames = waveRead.getnframes()
            self.numChannels = waveRead.getnchannels()
            self.sampleWidth = waveRead.getsampwidth()
            self.sampleRate = waveRead.getframerate()
            self.buffer = bytearray(waveRead.readframes(self.numFrames))
            waveRead.close()
        elif isinstance(sound, int):
            self.filename = ''
            self.numFrames = sound
            self.numChannels = 1
            self.sampleWidth = int(self.NUM_BITS_PER_SAMPLE / 8)
            self.sampleRate = sampleRate
            numBytes = self.numChannels * self.numFrames * self.sampleWidth
            self.buffer = bytearray([0] * numBytes)
        elif isinstance(sound, Sound):
            self.filename = sound.filename
            self.numFrames = sound.numFrames
            self.numChannels = sound.numChannels
            self.sampleWidth = sound.sampleWidth
            self.sampleRate = sound.sampleRate
            self.buffer = sound.buffer.copy()
        self.playbacks = []

    def __str__(self):
        """Return string representation of this sound

        Returns
        -------
        str
            representation of this sound
        """
        output = "Sound file: {} number of samples: {}".format(self.filename, self.numFrames)
        return output

    def __repr__(self):
        """Return string representation of this sound

        Returns
        -------
        str
            representation of this sound
        """
        return self.__str__()

    # ----------------------- accessors --------------------------------------

    def getBuffer(self):
        """Returns this sound's associated buffer

        Returns
        -------
        arr
            The sounds buffer(i.e. an array representaion of the sound)
        """
        return self.buffer

#    def getAudioFileFormat(self):
#        return self.audioFileFormat # not yet defined

    def getSamplingRate(self):
        """Return this sound's sampling rate

        Returns
        -------
        int
            the sampling rate
        """
        return self.sampleRate

    def getSoundExplorer(self):
        return "The sound explore tool is not implemented yet for jes4py"
        #return self.soundExplorer
        # not yet defined

    def asArray(self):
        """Returns an array representation of the sound

        Returns
        -------
        arr
            The sounds buffer(i.e. an array representaion of the sound)
        """
        return self.buffer

    def getPlaybacks(self):
        """Returns an array of all the current sound's playbacks

        Returns
        -------
        list
            array of all the current sounds playbacks
        """
        return self.playbacks

    def getChannels(self):
        """Return sounds number of channels

        Returns
        -------
        int
            number of channels
        """
        return self.numChannels
        
    def getFileName(self):
        """Return sounds file name

        Returns
        -------
        string
            name of associated file
        """
        return self.filename

    def getFileName(self):
        """Return sounds file name

        Returns
        -------
        string
            name of associated file
        """
        return self.filename

    def getFrame(self, frameNum):
        """Obtains all the data from a specified frame in the audio data

        Parameters
        ----------
        frameNum : int
            the index of the frame to access

        Returns
        -------
        bytearray
            the array containing all of the bytes in frame
        """
        if (frameNum >= self.numFrames):
            print("The index {}, does not exist. The last valid index is {}".format(frameNum, self.numFrames-1))
            
        frameSize = int(len(self.buffer)/self.numFrames)
        theFrame = bytearray(frameSize)
        for i in range(frameSize):
            theFrame[i] = self.buffer[frameNum * frameSize + i]
        return theFrame

    # ----------------------- modifiers --------------------------------------

    def setBuffer(self, newBuffer):
        """Changes the buffer assoiciated with the current sound to the (newBuffer)

        Parameters
        ----------
        newBuffer : int
            the length of the buffer that will replace (self.buffer)
        newBuffer : bytearray
            the byte array that (self.buffer) is being replaced with
        """
        if isinstance(newBuffer, int):
            self.buffer = bytearray(newBuffer)
        elif isinstance(newBuffer, bytearray):
            self.buffer = newBuffer.copy() #maybe not a copy?

    # def setAudioFileFormat(self, audioFileFormat):
    #     self.audioFileFormat = audioFileFormat
    
    def setSoundExplorer(self, soundExplorer):
        #self.soundExplorer = soundExplorer
        print("The sound explorer tool is not yet properly implemented for jes4py")

    # ------------------------ methods ---------------------------------------

    def makeAIS(self):
        print('This function is not implemented in jes4py.')

    def isStereo(self):
        """Method to check if a sound is stereo (2 channel) or not

        Returns
        -------
        bool
            True if stereo else False
        """
        return self.numChannels != 1

    def play(self):
        """Play a sound - nonblocking
        """
        waveObject = sa.WaveObject(self.buffer, self.numChannels, self.sampleWidth, self.sampleRate)
        self.playbacks.append(waveObject.play())

    def explore(self):
        """Open a sound explorer on a copy of this sound
        """
        # Make a new sound explorer
        # Open it with a copy of this sound
        print("The sound explore tool is not yet implemented in jes4py")

    def playNote(self, key, duration, intensity):
        print("This method is not implemented in jes4py.")

    def convert(self, mp3File, wavFile):
        print("this method isn't implemented.")
        # try:
        #     fi = open(mp3File, "r")
        #     contents = fi.read()
        #     fi.close()
        #     writeFi = open(wavFile, "w")
        #     writeFi.write(contents)
        #     writeFi.close()
        #     print("this method isn't implemented yet.")
        # except Exception:
        #     print("Couldn't covert the file {}".format(mp3File))

    def blockingPlay(self):
        """Play a sound - blocking
        """
        self.play()
        self.playbacks[-1].wait_done()

    def stopPlaying(self):
        """Stop playback of all currently playing sounds
        """
        while len(self.playbacks) > 0:
            self.playbacks.pop().stop()

    def removePlayback(self, playbackToRemove):
        """Method to remove a playback from the list of playbacks
        
        Parameters
        ----------
        playbackToRemove : PlayObject
            the playback that we want to remove

        """
        if (self.playbacks.contains(playbackToRemove)):
            self.playbacks.remove(playbackToRemove)
            playbackToRemove = None

    def getLengthInFrames(self):
        """Obtains number of sample frames in the audio data

        Returns
        -------
        int
            the number of sample frames of audio data in the sound
        """
        return self.numFrames

    def getNumSamples(self):
        """Returns the number of samples in this sound

        Returns
        -------
        int
            the number of sample frames
        """
        return self.getLengthInFrames()

    def getSample(self, frameNum):
        """Create and return a SoundSample object for the given frame number

        Parameters
        ----------
        frameNum : int
            the frame from which to retrieve the SoundSample object

        Returns
        -------
        SoundSample
            a SoundSample object for this frame number
        """
        return SoundSample(self, frameNum)

    def getSamples(self):
        """Method to create and return an array of SoundSample objects

        Returns
        -------
        list of SoundSample
            the array of SoundSample objects
        """
        samples = []
        for i in range(self.numFrames):
            samples.append(SoundSample(self, i))
        return samples

    def reportIndexException(self, index):
        """Method to report an index exception for this sound

        Parameters
        ----------
        index : int
            the index
        """
        print("The index {} isn't valid for this sound".format(index))

    def getSampleValueAt(self, index):
        """Get the sample at the passed index and handle any SoundExceptions

        Parameters
        ----------
        index : int
            the desired index

        Returns
        -------
        int
            the sample value
        """
        try:
            value = self.getSampleValue(index)
        except:
            reportIndexException(index)
        return value
    
    def getSampleValue(self, frameNum):
        """Sets the value of the sample at the indicated frame
        
        If this is a mono sound, obtains the single sample contained
        within this frame, else obtains the first (left) sample
        contained in the specified frame.

        Parameters
        ----------
        frameNum : int
            the index of the frame to access

        Returns
        -------
        int
            integer representation of the bytes contained within frame
        """
        n = frameNum * self.sampleWidth * self.numChannels
        m = n + self.sampleWidth
        return int.from_bytes(self.buffer[n:m], byteorder='little', signed=True)    

    def getLeftSample(self, frameNum):
        """Obtains the left sample contained at the specified frame

        Parameters
        ----------
        frameNum : int
            the index of the frame to access

        Returns
        -------
        int
            integer representation of the bytes contained in the specified frame.
        """
        if not self.isStereo():
            print("Sound is not stereo, cannot access left value")
        return self.getSampleValue(frameNum)

    def getRightSample(self, frameNum):
        """Obtains the right sample contained at the specified frame

        Parameters
        ----------
        frameNum : int
            the index of the frame to access

        Returns
        -------
        int
            integer representation of the bytes contained in the specified frame.
        """
        if not self.isStereo():
            print("Sound is not stereo, cannot access right value")
        else:
            n = frameNum * self.sampleWidth * self.numChannels + self.sampleWidth
            m = n + self.sampleWidth
            return int.from_bytes(self.buffer[n:m], byteorder='little', signed=True)    

    def getLengthInBytes(self):
        """Obtains the length of this sound in bytes
        
        Note, that this number is not neccessarily the same as the length of
        this sound's file in bytes

        Returns
        -------
        int
            the sound length in bytes
        """
        return len(buffer)

    def getLength(self):
        """Return the length of the sound as the number of samples

        Returns
        -------
        int
            the length of the sound as the number of samples
        """
        return self.getNumSamples()

    def setFrame(self, frameNum, frame):
        """Changes the value of each byte of the specified frame

        Parameters
        ----------
        frameNum : int
            the index of the frame to change
        frame : byte array
            the bytes that will be copied into this sound's buffer
        """
        if frameNum >= self.numFrames:
            print("The frame number {} does not exist".format(frameNum))
            print("The last valud frame number is {}".format(self.numFrames-1))
        else:
            for i in range(self.sampleWidth):
                self.buffer[frameNum * self.sampleWidth + i] = frame[i]

    def setSampleValueAt(self, index, value):
        """Method to set the sample value at the specified index

        Parameters
        ----------
        index : int
            the index
        value : int or float
            the new value
        """
        try:
            self.setSampleValue(index, int(value))
        except:
            reportIndexException(index)

    def setSampleValue(self, frameNum, value):
        """Sets the value of the sample found at the specified frame
        
        If this sound has more than one channel, then we default to setting
        only the first (left) sample.  Values outside of the range
        [MAX_NEG, MAX_POS] are silently clipped to be within that range.
    
        Parameters
        ----------
        frameNum : int
            the index of the frame where the sample should be changed
        value : int
           the new sample value
        """
        n = frameNum * self.sampleWidth * self.numChannels
        m = n + self.sampleWidth
        value = max(min(value, self.MAX_POS), self.MAX_NEG)
        self.buffer[n:m] = value.to_bytes(self.sampleWidth,
                                          byteorder='little',
                                          signed=True)

    def setLeftSample(self, frameNum, value):
        """Set the left sample value in a stereo sample
        
        Parameters
        ----------
        frameNum : int
            the index of the frame where the sample should be changed
        value : int
            an integer representation of the new sample
        """
        if not self.isStereo():
            print("Sound is not stereo, cannot set left value")
        else:
            self.setSampleValue(frameNum, value)

    def setRightSample(self, frameNum, value):
        """Set the right sample value in a stereo sample
        
        Values outside of the range [MAX_NEG, MAX_POS] are silently clipped
        to be within that range.

        Parameters
        ----------
        frameNum : int
            the index of the frame where the sample should be changed
        value : int
            an integer representation of the new sample
        """
        if not self.isStereo():
            print("Sound is not stereo, cannot set right value")
        else:
            n = frameNum * self.sampleWidth * self.numChannels + self.sampleWidth
            m = n + self.sampleWidth
            value = max(min(value, self.MAX_POS), self.MAX_NEG)
            self.buffer[n:m] = value.to_bytes(self.sampleWidth,
                                              byteorder='little',
                                              signed=True)

    def setFileName(self, filename):
        """Set sound's file name

        Parameters
        ----------
        filename : string
            filename to assign to this sound
        """
        self.filename = filename

    # ------------------------ File I/O ---------------------------------------

    def loadFromFile(self, inFileName):
        """Resets the fields of this sound so that it now represents the
           sound in the specified file.  If successful, the fileName variable
           is updated such that it is equivalent to the passed in file
 
        Parameters
        ----------
        inFileName : str
            the name of the file to read the sound in from
        """
        self.filename = inFileName
        waveRead = wave.open(self.filename, 'rb')
        self.numFrames = waveRead.getnframes()
        self.numChannels = waveRead.getnchannels()
        self.sampleWidth = waveRead.getsampwidth()
        self.sampleRate = waveRead.getframerate()
        self.buffer = bytearray(waveRead.readframes(self.numFrames))
        waveRead.close()

    def write(self, fileName):
        """Write the sound to a wav file and throw an error if it can't be written
 
        Parameters
        ----------
        fileName : str
            the name of the file to write the sound to
        """
        try:
            self.writeToFile(fileName)
        except IOError:
            print("Couldn't write file to " + fileName)

    def writeToFile(self, outFileName):
        """Write the sound to a wav file
 
        Parameters
        ----------
        outFileName : str
            the name of the file to write the sound to
        """
        # Append the .wav extension if the filename doesn't have it
        # (Uncomment the next two lines to implement this functionality)
        # if not outFileName.endswith(".wav"):
        #     outFileName = outFileName+".wav"

        file = wave.open(outFileName, "wb")
        file.setnframes(self.numFrames)
        file.setnchannels(self.numChannels)
        file.setsampwidth(self.sampleWidth)
        file.setframerate(self.sampleRate)
        file.writeframes(self.buffer)
        file.close()



"""Module for choosing and setting file and directory names

   Based on jes/jes/java/FileChooser.java
"""

import os
import wx
import os, sys, subprocess
from jes4py import Config

def pickAFile():
    """Method to let the user pick a file and return the full name as
       as a string.  If the user didn't pick a file then the name
       will be None.

    Returns:
        the file file name of the picked file or None
    """
    # Create open file dialog
    directory = Config.getConfigVal('CONFIG_SESSION_PATH')
    scriptpath = os.path.join(Config.getConfigVal("CONFIG_JES4PY_PATH"),
            'filePicker.py')
    path = subprocess.check_output([sys.executable, scriptpath, 'file',
            directory]).decode()
    if path == '':
        return None
    else:
        Config.setConfigVal('CONFIG_SESSION_PATH', os.path.dirname(path))
        return path

def pickADirectory():
    """Method to let the user pick a directory and return the full
       path as a string.

    Returns:
        the full directory path
    """
    # Create open file dialog
    directory = Config.getConfigVal('CONFIG_SESSION_PATH')
    scriptpath = os.path.join(Config.getConfigVal("CONFIG_JES4PY_PATH"),
            'filePicker.py')
    path = subprocess.check_output([sys.executable, scriptpath, 'folder',
            directory]).decode()
    if path == '':
        return None
    else:
        Config.setConfigVal('CONFIG_SESSION_PATH', path)
        return path

def getMediaPath(fileName):
    """Method to get full path for the passed file name

    Parameters:
        fileName - the name of a file
    Returns:
        the full path for the file
    """
    return os.path.join(Config.getConfigVal("CONFIG_MEDIA_PATH"), fileName)

def getMediaDirectory():
    """Method to get the directory for the media

    Returns:
        the media directory
    """
    return Config.getConfigVal("CONFIG_MEDIA_PATH")

def setMediaPath(directory):
    """Method to set the media path by setting the directory to use

    Parameters:
        directory - the directory to use for the media path
    """
    Config.setConfigVal("CONFIG_MEDIA_PATH", directory)

def pickMediaPath():
    path = pickADirectory()
    Config.setConfigVal("CONFIG_MEDIA_PATH", path)



from jes4py import Config
import math
import wx
import os, sys, subprocess

class Pixel:
    """Provides access to pixels within an PIL image

    Attributes
    ----------
    wrapLevels : boolean
        Indicates whether levels outside the range 0-255 are clamped
        or wrapped around (saturating or modular arithmetic).
        False to clamp levels, true to modulo them.
    """

    wrapLevels = False

    def __init__(self, image=None, x=None, y=None):
        """Pixel constructor

        Parameters
        ----------
        image : PIL.Image
            image the pixel belongs to
        x : int
            column of the pixel
        y : int
            row of the pixel
        """
        self.wrapLevels = Config.getConfigVal("CONFIG_WRAPPIXELVALUES")
        self.image = image
        self.x = x
        self.y = y
        #self.color = color

    def __str__(self):
        """Return string with pixel contents

        Returns
        -------
        str
            user-readable pixel information
        """
        rgb = self.image.getpixel((self.x, self.y))
        return "Pixel red={} green={} blue={}".format(rgb[0], rgb[1], rgb[2])

    def __repr__(self):
        """Return string representation of pixel

        Returns
        -------
        str
            string of pixel contents
        """
        return self.__str__()

    def getX(self):
        """Gets column containing pixel

        Returns
        -------
        int
           column containing the pixel
        """
        return self.x

    def getY(self):
        """Gets row containing pixel

        Returns
        -------
        int
           row containing the pixel
        """
        return self.y

    def getAlpha(self):
        """Return alpha level in pixel (NOT IMPLEMENTED)

        Returns
        -------
        int
            alpha level in pixel
        """
        return 255

    def getRed(self):
        """Return red level in pixel

        Returns
        -------
        int
            red level in pixel
        """
        rgb = self.image.getpixel((self.x, self.y))
        return rgb[0]

    def getGreen(self):
        """Return green level in pixel

        Returns
        -------
        int
            green level in pixel
        """
        rgb = self.image.getpixel((self.x, self.y))
        return rgb[1]

    def getBlue(self):
        """Return blue level in pixel

        Returns
        -------
        int
            blue level in pixel
        """
        rgb = self.image.getpixel((self.x, self.y))
        return rgb[2]

    def getAverage(self):
        """Return the average of the color values of this pixel

        Returns
        -------
        int
            rounded average of red, green, and blue pixel values
        """
        rgb = self.image.getpixel((self.x, self.y))
        return round((rgb[0] + rgb[1] + rgb[2]) / 3.0)
    
    def setAlpha(self, value):
        """Set alpha level in the pixel (NOT IMPLEMENTED)

        Parameters
        ----------
        value : int
            alpha level for pixel
        """
        value = 0
        # do nothing, as we have not implemented alpha values

    def setRed(self, value):
        """Set red level in the pixel

        Parameters
        ----------
        value : int
            red level for pixel
        """
        value = Pixel.correctLevel(value)
        color = Color(self.image.getpixel((self.x, self.y)))
        newColor = (value, color.getGreen(), color.getBlue())
        self.image.putpixel((self.x, self.y), newColor)

    def setGreen(self, value):
        """Set green level in the pixel

        Parameters
        ----------
        value : int
            green level for pixel
        """
        value = Pixel.correctLevel(value)
        color = Color(self.image.getpixel((self.x, self.y)))
        newColor = (color.getRed(), value, color.getBlue())
        self.image.putpixel((self.x, self.y), newColor)

    def setBlue(self, value):
        """Set blue level in the pixel

        Parameters
        ----------
        value : int
            blue level for pixel
        """
        value = Pixel.correctLevel(value)
        color = Color(self.image.getpixel((self.x, self.y)))
        newColor = (color.getRed(), color.getGreen(), value)
        self.image.putpixel((self.x, self.y), newColor)

    def colorDistance(self, testColor):
        """Computes the Euclidean distance norm between this pixel and a color

        Parameters
        ----------
        testColor : Color
            the color to compute the distance to

        Returns
        -------
        float
            the Euclidean distance between the two colors
        """
        r = pow((self.getRed() - testColor.getRed()), 2)
        g = pow((self.getGreen() - testColor.getGreen()), 2)
        b = pow((self.getBlue() - testColor.getBlue()), 2)
        return math.sqrt(r + g + b)

    def getColor(self):
        """Returns the color object for the pixel

        Returns
        -------
        Color
            color object for the pixel
        """
        return Color(self.image.getpixel((self.x, self.y)))

    def setColor(self, color):
        """Set the color of a pixel

        Parameters
        ----------
        color : Color
            color to assign to pixel
        """
        self.image.putpixel((self.x, self.y), color.getRGB())

    def setColorFrom(self, otherPixel):
        """Set color of this pixel using color value from otherPixel

        Parameters
        ----------
        otherPixel : Pixel
            pixel to get color values from
        """
        self.setColor(otherPixel.getColor())

    #def updatePicture(self, alpha, red, green, blue):
        """Update the picture based on the passed color values for this pixel

        **** 2020-06-28 WHAT IS THIS FUNCTION SUPPOSED TO DO? ****

        Parameters
        ----------
        alpha : int
            transparency value (CURRENTLY IGNORED)
        red : int
            red color value
        green : int
            green color value
        blue : int
            blue color value
        """
        #self.setAlpha(alpha)
        #self.setRed(red)
        #self.setGreen(green)
        #self.setBlue(blue)

    @classmethod
    def correctLevel(cls, level):
        """Map color to [0..255] according to the wrapLevels setting

        Parameters
        ----------
        level : int
            nonnegative integer representing a color value

        Returns
        -------
        int
            corrected color level
        """
        level = int(level)
        if cls.wrapLevels:
            return level % 256
        elif level < 0:
            return 0
        elif level > 255:
            return 255
        return level

    @classmethod
    def setWrapLevels(cls, doWrap):
        """Changes Pixel's behavior for dealing with levels outside [0..255]

        Parameters
        ----------
        doWrap : boolean
            true to cause color level wrapping, false for level truncation
        """
        cls.wrapLevels = doWrap

    @classmethod
    def getWrapLevels(cls):
        """Return Pixel's behavior for dealing with levels outside [0..255]

        Returns
        -------
        boolean
            true means levels are wrapped, false means levels are truncated
        """
        return cls.wrapLevels




class Color:
    """Class for storing and doing computations with colors and RGB values

    Has getters and setters for the red, green, and blue portions of
    the over all color as well as giving the user capabilities to
    compare color values.
    """

    def __init__(self, r, g=None, b=None):
        """Initialize a color object

        Parameters
        ----------
        r : list, tuple, Color or int
            If r is a list or a tuple then r contains the RGB values,
            if r is a Color object then use its color list for RGB values,
            if r is an int then use it for all colors (graylevel)
        g : int
            green level (if not provided then r is used)
        b : int
            blue level (if not provided then r is used)
        """

        if b == None or g == None:
            if isinstance(r, tuple):
                self.color = r
            elif isinstance(r, list):
                self.color = tuple(r)
            elif isinstance(r, Color):
                self.color = r.getRGB()
            else:
                val = Pixel.correctLevel(r)
                self.color = (val, val, val)
        else:
            r = Pixel.correctLevel(r)
            g = Pixel.correctLevel(g)
            b = Pixel.correctLevel(b)
            self.color = (r, g, b)

    def __str__(self):
        """String for Color

        Returns
        -------
        str
            user-readable color information
        """
        return "color r={} g={} b={}".format(self.color[0], self.color[1], self.color[2])
    
    def __repr__(self):
        """Representation of color

        Returns
        -------
        str
            string of color contents
        """
        return "Color({}, {}, {})".format(self.color[0], self.color[1], self.color[2])
    
    def __eq__(self, otherColor):
        """Test for equality between two color objects

        Parameters
        ----------
        otherColor : Color
            color to compare

        Returns
        -------
        boolean
            True if colors are the same, False otherwise
        """
        return (self.color == otherColor.color)

    def __ne__(self, otherColor):
        """Test for inequality between two color objects

        Parameters
        ----------
        otherColor : Color
            color to compare

        Returns
        -------
        boolean
            True if colors are not the same, False otherwise
        """
        return (not self.__eq__(otherColor))

    def __add__(self, otherColor):
        """Sum of this color and otherColor

        Parameters
        ----------
        otherColor : Color
            the color to add
        
        Returns
        -------
        Color
            the sum of the two colors, mapped to [0..255]
        """
        if isinstance(otherColor, Color):
            r = Pixel.correctLevel(self.color[0] + otherColor.color[0])
            g = Pixel.correctLevel(self.color[1] + otherColor.color[1])
            b = Pixel.correctLevel(self.color[2] + otherColor.color[2])
        return Color(r, g, b)

    def __sub__(self, otherColor):
        """Difference of this color and otherColor
        
        Parameters
        ----------
        otherColor : Color
            the color to subtract
        
        Returns
        -------
        Color
            the difference of the two colors, mapped to [0..255]
        """
        r = Pixel.correctLevel(self.color[0] - otherColor.color[0])
        g = Pixel.correctLevel(self.color[1] - otherColor.color[1])
        b = Pixel.correctLevel(self.color[2] - otherColor.color[2])
        return Color(r, g, b)

    def setRGB(self, r, g, b):
        """Sets this color's red, green, blue color values
    
        Parameters
        ----------
        r, g, b : int or float
            the red, green, and blue values
        """
        r = Pixel.correctLevel(r)
        g = Pixel.correctLevel(g)
        b = Pixel.correctLevel(b)
        self.color = (r, g, b)

    def getRGB(self):
        """Returns the colors RGB values in a tuple 

        Returns
        -------
        tuple of int
            tuple of red, green, and blue values
        """
        return self.color

    def getRed(self):
        """Returns the color's red value

        Returns
        -------
        int
            the red value
        """
        return self.color[0]

    def getGreen(self):
        """Returns the color's green value

        Returns
        -------
        int
            the green value
        """
        return self.color[1]

    def getBlue(self):
        """Returns the color's blue value

        Returns
        -------
        int
            the blue value
        """
        return self.color[2]

    def distance(self, otherColor):
        """Computes the Euclidean distance norm between two colors

        Parameters
        ----------
        otherColor : Color
            the color to compute the distance to

        Returns
        -------
        float
            the Euclidean distance between the two colors
        """
        if isinstance(otherColor, Color):
            r = pow((self.color[0] - otherColor.color[0]), 2)
            g = pow((self.color[1] - otherColor.color[1]), 2)
            b = pow((self.color[2] - otherColor.color[2]), 2)
            return math.sqrt(r + g + b)
        else:
            print("distance() expects a Color object")
            return

    def scaleColor(self, scaleFactor):
        """Return a uniformly scaled version of this color

        Parameters
        ----------
        scaleFactor : float
            the factor to scale each color component by

        Returns
        -------
        Color
            the color with the scaled components
        """
        r = Pixel.correctLevel(self.color[0] * scaleFactor)
        g = Pixel.correctLevel(self.color[1] * scaleFactor)
        b = Pixel.correctLevel(self.color[2] * scaleFactor)
        return Color(r, g, b)

    def makeDarker(self):
        """Return a darker version of this color

        Returns
        -------
        Color
            darker version of this color
        """
        return self.scaleColor(7.0/10.0)

    def makeLighter(self):
        """Return a lighter version of this color

        Returns
        -------
        Color
            lighter version of this color
        """
        if self.color == (0,0,0):
            # Special case -- black gets lighted to very dark gray
            lighterColor = Color(3,3,3)
        else:
            # Scale color values by 10/7
            lighterColor = self.scaleColor(10.0/7.0)
            if max(lighterColor.color) <= 2:
                # if all color values are 2 or less we need to adjust them
                c = list(lighterColor.color)
                for i in range(3):
                    if c[i] > 0 and c[i] < 2:
                        c[i] += 3
                    elif c[i] > 0 and c[i] == 2:
                        c[i] += 2
                lighterColor = Color(c)
        return lighterColor

    # @classmethod
    # def pickAColor(cls):
    #     app = wx.App()
    #     dlg = wx.ColourDialog(wx.GetApp().GetTopWindow())
    #     color = None
    #     if dlg.ShowModal() == wx.ID_OK:
    #         red =  dlg.GetColourData().GetColour().Red()
    #         green = dlg.GetColourData().GetColour().Green()
    #         blue = dlg.GetColourData().GetColour().Blue()
    #         color = Color(red,green,blue)
    #     return color

    @classmethod
    def pickAColor(cls):
        # Start subprocess using current Python intepreter to run a script
        scriptpath = os.path.join(Config.getConfigVal("CONFIG_JES4PY_PATH"),
            'colorPicker.py')
        color = subprocess.check_output([sys.executable, scriptpath]).decode()
        if color == '':
            color = None
        else:
            color = Color(list(map(int,color.split())))
        return color



#!/usr/bin/env python3

"""
show.py - script program to implement the "show" and "repaint" functionality
            in JES for the JES4py package

Written: 2020-07-22 Jonathan Senning <jonathan.senning@gordon.edu>
Revised: 2020-09-02 Jonathan Senning <jonathan.senning@gordon.edu>
- received pickled picture objects rather than filename, no need to read files

The "show()" function in JES will open a new window and display the image
associated with the given Picture object in the window.  The window is
static and does not provide for user interaction.  When the "repaint()"
function is called, however, the contents of the window are refreshed so
that any changes to the displayed Picture (the image itself or the picture's
title) are displayed.  Thus, repaint() can be used to produce simple
animations.

The jes4py.Picture module defines the Picture class, which has show() and
repaint() methods.  The show() method causes this script to be run in a
subprocess and uses a pipe to send a pickled picture object to the subprocess.
The repaint() method checks to make sure a subprocess for this picture object
is currently running and then sends the pickled updated picture object.

This script expects the initial byte of data to be 0 (to exit) or 1
(a pickled picture object follows).

Implementation note: The thread portion of this program is based on the
first example at https://wiki.wxpython.org/LongRunningTasks.  The pickling
method is based on that shown in
https://www.imagetracking.org.uk/2018/03/piping-numpy-arrays-to-other-processes-in-python/
"""

import wx
import sys, os
import pickle
from threading import *
from jes4py import *

class MessageEvent(wx.PyEvent):
    """Simple event to carry arbitrary result data"""
    def __init__(self, data):
        """Initializer for MessageEvent class

        Parameters
        ----------
        data : (can be any type, but will be a string in this program)
            the message contents
        """
        wx.PyEvent.__init__(self)
        self.SetEventType(wx.ID_ANY)
        self.data = data

# Thread class that executes processing
class Listener(Thread):
    """Listener Thread Class"""
    def __init__(self, notifyWindow):
        """Initializer for Listener Thread Class

        Parameters
        ----------
        notifyWindow : wx.Frame
            the window to notify when a message is received
        """
        Thread.__init__(self)
        self.notifyWindow = notifyWindow
        self.start()

    def run(self):
        """Run Listener thread"""
        while True:
            # wait for control code
            data = sys.stdin.buffer.read(1)
            if data == Picture.show_control_exit:
                # shutdown program
                wx.PostEvent(self.notifyWindow, MessageEvent(None))
                return
            elif data == Picture.show_control_data:
                # read picture size and pickled picture data
                try:
                    data = sys.stdin.buffer.read(8)
                    dataLen = int.from_bytes(data, byteorder='big')
                    pkg = sys.stdin.buffer.read(dataLen)
                    wx.PostEvent(self.notifyWindow, MessageEvent(pkg))
                except RuntimeError:
                    return
            else:
                # unrecognised control code
                return

class MainWindow(wx.Frame):
    """Window class for show program
    """

    def __init__(self, parent):
        """Initializer for MainWindow

        Parameters
        ----------
        parent : wxFrame
            the parent frame
        """
        super(MainWindow, self).__init__(parent=parent)

        # Set up listener for data coming in over pipe
        self.Connect(-1, -1, wx.ID_ANY, self.OnMessage)
        self.worker = Listener(self)

        # Create panel for displayed window
        self.panel = wx.Panel(parent=self)
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.panel, 0, wx.ALIGN_LEFT|wx.ALIGN_TOP|wx.ALL, 0)
        self.SetSizerAndFit(self.sizer)

    def OnMessage(self, event):
        """Handle received message

        Parameters
        ----------
        event : wx.Event
            the event object

        event.data is either None (to indicate request to terminate program)
        or a pickled Picture object
        """
        if event.data is None:
            # all done
            self.Close()
        else:
            # unpickle data and update displayed image
            picture = pickle.loads(event.data)
            self.updateBitmap(picture)

    def updateBitmap(self, picture):
        """Update bitmap of displayed image

        Parameters
        ----------
        picture : Picture object
            picture to display
        """
        image = picture.getWxImage()
        imageSize = image.GetSize()
        bmp = wx.Bitmap(image)
        self.SetTitle(picture.getTitle())
        self.bitmap = wx.StaticBitmap(parent=self.panel, size=imageSize, \
                                        bitmap=bmp)
        self.SetClientSize(imageSize)
        self.Refresh()

# ===========================================================================
# Main program
# ===========================================================================

def main(argv):
    app = wx.App(False)
    frame = MainWindow(parent=None)
    frame.Show()
    app.MainLoop()

if __name__ == '__main__':
    main(sys.argv)



class Sample:
    def __init__(self, sound, index):
        self.sound = sound
        self.index = index

    def __str__(self):
        try:
            s = "Sample at {} with value {}".format(self.index, self.getValue())
        except:
            s = "Sample at {} with value {} value unknown".format(self.index)
        return s

    def __repr__(self):
        return self.__str__()

    def getSound(self):
        """Method to get this sample's sound object

        Returns
        -------
        Sound
           the sound object
        """
        return self.sound

    def getValue(self):
        """Method to get the sample value

        Returns
        -------
        int
            the sample's value
        """
        return self.sound.getSampleValueAt(self.index)

    def setValue(self, newValue):
        """Method to set the sample value

        Parameters
        ----------
        newValue : int or float
            the new value to store
        """
        self.sound.setSampleValueAt(self.index, int(newValue))



import os, sys
import wx
import atexit
import subprocess, tempfile, pickle
from subprocess import PIPE
import PIL.ImageDraw, PIL.Image
from jes4py import Config
from jes4py.PixelColor import Pixel, Color
from jes4py import FileChooser

class Picture:

    filename = None
    title = None
    extension = ".jpg"
    _PictureIndexOffset = 0
    tmpfilename = None
    process = None
    subprocessList = []
    show_control_exit = bytes([0])
    show_control_data = bytes([1])

    def __init__(self, *args, **kwargs):
        """Initializer for Picture class
        """
        self.filename = self.title = 'None'
        if len(args) == 0:
            # no parameters, make 100x200 picture with white background
            self.image = PIL.Image.new("RGB", (200, 100), (255, 255, 255))
        elif len(args) == 1:
            if isinstance(args[0], str):
                self.filename = self.title = args[0]
                # Assume 'we've been passed a filename
                if not os.path.isfile(self.filename):
                    # File not found, try prepending media path
                    filepath = FileChooser.getMediaPath(self.filename)
                    if os.path.isfile(filepath):
                        self.filename = self.title = filepath
                try:
                    self.image = PIL.Image.open(self.filename)
                except:
                    self.image = PIL.Image.new("RGB", (600, 200))
                    draw = PIL.ImageDraw.Draw(self.image)
                    draw.text((0, 100), "Couldn't load " + self.filename)
            elif isinstance(args[0], Picture):
                # We've been passed a Picture object
                self.image = args[0].image.copy()
                self.filename = args[0].filename
                self.title = args[0].title
            elif isinstance(args[0], PIL.Image.Image):
                # We've been passed a PIL image object
                self.image = args[0]
                try:
                    self.filename = self.title = args[0].filename
                except AttributeError:
                    pass
        elif len(args) == 2 or len(args) == 3:
            # We've been passed width and height, and possibly a color
            size = (int(args[0]), int(args[1]))
            c = Color(255,255,255) if len(args) == 2 else args[2]
            self.image = PIL.Image.new("RGB", size, c.color)
        else:
            print("Could not construct Picture object")

    def __str__(self):
        """Return string representation of this picture

        Returns
        -------
        str
            representation of this picture
        """
        output = "Picture, filename {} height {} width {}".format(
            self.filename, self.image.height, self.image.width)
        return output

    def __repr__(self):
        """Return string representation of this picture

        Returns
        -------
        str
            representation of this picture
        """
        return self.__str__()

    def getExtension(self):
        """Return the filename extension for this picture

        Returns
        -------
        str
            the file extension for the picture (indicates the image file format)
        """
        return self.extension

    def copyPicture(self, sourcePicture):
        """Copies the passed in picture to the current Picture

        Parameters
        ----------
        sourcePicture : Picture
            the Picture object that self will look like

        """
        im = PIL.Image.new("RGB", (self.getWidth(), self.getHeight()), (0,0,0))
        im = sourcePicture.getImage().copy()
        self.setImage(im)

    def setAllPixelsToAColor(self, acolor):
        """Makes the image associated with the picture filled in with one color

        Parameters
        ----------
        acolor : Color
            the color that outlines arc
        """
        if not isinstance(acolor, Color):
            print ("setAllPixelsToAColor(color): Input is not a color")
            raise ValueError
        self.image = PIL.Image.new("RGB", (self.getWidth(), self.getHeight()), acolor.getRGB())

    def getFileName(self):
        """Return picture file name

        Returns
        -------
        string
            name of file containing picture data
        """
        return self.filename

    def setFileName(self, filename):
        """Set picture file name

        Parameters
        ----------
        filename : string
            filename to assign to this picture
        """
        self.filename = filename

    def getTitle(self):
        """Return picture title

        Returns
        -------
        str
            picture title
        """
        return self.title
        
    def setTitle(self, title):
        """Set picture title

        Parameters
        ----------
        title : str
            title to assign to this picture
        """
        self.title = title

    def getWidth(self):
        """Return the width of this image in this picture

        Returns
        -------
        int
            number of pixels in a row of this image
        """
        return self.image.width

    def getHeight(self):
        """Return the height of the image in this picture

        Returns
        -------
        int
            number of pixels in a column of this image
        """
        return self.image.height

    def getImage(self):
        """Return the PIL Image associated with this picture

        Returns
        -------
        PIL.Image.Image
            the PIL Image associated with this picture
        """
        return self.image

    def setImage(self, image):
        """Sets the PIL Image associated with this picture

        Parameters
        ----------
        image : PIL.Image.Image
            the PIL Image to associate with this picture
        """
        self.image = image

    def getBasicPixel(self, x, y):
        """Return the pixel at specified coordinates as a tuple.

        Parameters
        ----------
        x, y : int
            the coordinates of the pixel

        Returns
        -------
        tuple of int
            the color of the pixel at position (x,y) as a tuple
        """
        return self.getPixel(x,y).getColor().getRGB()

    def setBasicPixel(self, x, y, rgb):
        """Sets the pixel at specified coordinates to a color based on rgb(tuple)

        Parameters
        ----------
        x, y : int
            the coordinates of the pixel
        rgb : tuple of int
            the color the pixel will be set to
        """
        col = Color(rgb[0], rgb[1], rgb[2])
        self.getPixel(x,y).setColor(col)

    def getPixel(self, x, y):
        """Return the pixel at specified coordinates

        Parameters
        ----------
        x, y : int
            the coordinates of the pixel

        Returns
        -------
        Pixel
            the pixel at (x,y) in this picture
        """
        pix = Pixel(self.image, x, y)
        return pix

    def getPixels(self):
        """Return list of pixels contained in picture

        Returns a list of all pixels in this picture as a flattened array.
        Pixels are listed row-by-row.

        Returns
        -------
        list of Pixel
            list of pixels in this picture
        """
        pixels = list()
        for y in range(self.image.height):
            for x in range(self.image.width):
                pixels.append(Pixel(self.image, x, y))
        return pixels

    def addLine(self, acolor, x1, y1, x2, y2):
        """Draw a line on this picture
    
        acolor : Color
            the color of the line
        x1 : int
            the x-coordinate of the first point
        y1 : int
            the y-coordinate of the first point
        x2 : int
            the x-coordinate of the second point
        y2 : int
            the y-coordinate of the second point
        """
        draw = PIL.ImageDraw.Draw(self.image)
        shape = [x1, y1, x2, y2]
        draw.line(shape, fill=acolor.getRGB())

    def addText(self, acolor, x, y, string):
        """Add a line of text to the picture
    
        Parameters
        ----------
        acolor : Color
            the color of the text
        x : int
            the x-coordinate of the top left corner of the text
        y : int
            the y-coordinate of the top left corner of the text
        string : str
            the text that will be drawn on the picture
        """
        draw = PIL.ImageDraw.Draw(self.image)
        # font = ImageFont.truetype(<font-file>, <font-size>)
        # font = ImageFont.truetype("sans-serif.ttf", 16)
        # draw.text((x, y),"Sample Text",(r,g,b))
        draw.text((x, y), string, acolor.getRGB())

    def addTextWithStyle(self, acolor, x, y, string, style):
        """Add text to a picture withe a particular font style

        Parameters
        ----------
        acolor : Color
            the color of the text
        x : int
            the x-coordinate of the top left corner of the text
        y : int
            the y-coordinate of the top left corner of the text
        string : str
            the text that will be drawn on the picture
        style : ???
            the font style to be used
        """
    #    draw = PIL.ImageDraw.Draw(self.image)
    #    # font = ImageFont.truetype(<font-file>, <font-size>)
    #    # font = ImageFont.truetype("sans-serif.ttf", 16)
    #    # draw.text((x, y),"Sample Text",(r,g,b))
    #    draw.text((x, y), string, acolor.getRGB())
        print("This function is not yet implemented.")

    def addRect(self, acolor, x, y, w, h):
        """Draw the outline of a rectangle on this picture
    
        acolor : Color
            the color of the rectangle border
        x : int
            the x-coordinate of the upper-left corner of the rectangle
        y : int
            the y-coordinate of the upper-left corner of the rectangle
        w : int
            the width of the rectangle
        h : int
            the height of the rectangle
        """
        draw = PIL.ImageDraw.Draw(self.image)
        shape = [x, y, x+w, y+h]
        draw.rectangle(shape, fill = None, outline = acolor.getRGB()) 

    def addRectFilled(self, acolor, x, y, w, h):
        """Draw a filled rectangle on this picture
    
        acolor : Color
            the color that the rectangle is filled
        x : int
            the x-coordinate of the upper-left corner of the rectangle
        y : int
            the y-coordinate of the upper-left corner of the rectangle
        w : int
            the width of the rectangle
        h : int
            the height of the rectangle
        """
        draw = PIL.ImageDraw.Draw(self.image)
        shape = [x, y, x+w, y+h]
        color = acolor.getRGB()
        draw.rectangle(shape, fill = color, outline = color) 

    def addOvalFilled(self, acolor, x, y, w, h):
        """Draw a filled oval on this picture
    
        acolor : Color
            the color that the oval is filled with.
        x : int
            the x-coordinate of the upper-left corner of the boundary rectangle for the oval
        y : int
            the y-coordinate of the upper-left corner of the boundary rectangle for the oval
        w : int
            the width of the oval
        h : int
            the height of the oval
        """
        draw = PIL.ImageDraw.Draw(self.image)
        shape = [x, y, x+w, y+h]
        color = acolor.getRGB()
        draw.ellipse(shape, fill=color, outline=color, width=1)

    def addOval(self, acolor, x, y, w, h):
        """Draw the outline of an oval on this picture
    
        acolor : Color
            the color of the oval border
        x : int
            the x-coordinate of the upper-left corner of the boundary rectangle for the oval
        y : int
            the y-coordinate of the upper-left corner of the boundary rectangle for the oval
        w : int
            the width of the oval
        h : int
            the height of the oval
        """
        draw = PIL.ImageDraw.Draw(self.image)
        shape = [x, y, x+w, y+h]
        draw.ellipse(shape, fill=None, outline=acolor.getRGB(), width=1)

    def addArcFilled(self, acolor, x, y, w, h, start, angle):
        """Draw a filled in arc on this picture
    
        acolor : Color
            the color that the arc is filled with
        x : int
            the x-coordinate of the center of the arc
        y : int
            the y-coordinate of the center of the arc
        w : int
            the width of the arc
        h : int
            the height of the arc
        start : int
            the start angle of the arc in degrees
        angle : int
            the angle of the arc relative to start in degrees
        """
        draw = PIL.ImageDraw.Draw(self.image)
        shape = [x, y, x+w, y+h]
        end = -start % 360
        start = -(start+angle) % 360
        if start > end:
            start, end = end, start
        color = acolor.getRGB()
        draw.pieslice(shape, start, end, fill=color, outline=color, width=1)

    def addArc(self, acolor, x, y, w, h, start, angle):
        """Draw the outline of an arc on this picture
    
        acolor : Color
            the color that outlines arc
        x : int
            the x-coordinate of the center of the arc
        y : int
            the y-coordinate of the center of the arc
        w : int
            the width of the arc
        h : int
            the height of the arc
        start : int
            the start angle of the arc in degrees
        angle : int
            the angle of the arc relative to start in degrees
        """
        draw = PIL.ImageDraw.Draw(self.image)
        shape = [x, y, x+w, y+h]
        end = -start % 360
        start = -(start+angle) % 360
        if start > end:
            start, end = end, start
        draw.arc(shape, start, end, fill=acolor.getRGB(), width=1)

    def copyInto(self, dest, upperLeftX, upperLeftY):
        """Returns a picture with the current picture copied into it

        Copies the pixels in the current picture into the dest picture
        starting at point (upperLeftX,upperLeftY)

        Parameters
        ----------
        dest : Picture
            the Picture that the current picture will be copied into
        upperLeftX : int
            the x-coord of the upper-left corner in dest where the current
            picture will be copied
        upperLeftY : int
            the y-coord of the upper-left corner in dest where the current
            picture will be copied

        Returns
        -------
        Picture
            the dest picture that has self copied into it
        """
        srcWidth = self.getWidth()
        srcHeight = self.getHeight()
        # # The next block of statments deal with cases where the self
        # # picture would extend past the rigth or bottom border of the
        # # destination picture. This is good, but an enhancement over
        # # what JES does.  To make it useful we should also allow for
        # # negative upperLeftX and upperLeftY values so the src picture
        # # could extend past the left or top border.  Until this is done
        # # we'll just leave this code disabled
        # dstWidth = dest.getWidth()
        # dstHeight = dest.getHeight()
        # if upperLeftX + srcWidth >= dstWidth:
        #     srcWidth = dstWidth - upperLeftX
        # if upperLeftY + srcHeight >= dstHeight:
        #     srcHeight = dstHeight - upperLeftY
        for x in range(0, srcWidth):
            for y in range(0, srcHeight):
                srcPix = self.getPixel(x, y)
                dstPix = dest.getPixel(x+upperLeftX, y+upperLeftY)
                dstPix.setColor(srcPix.getColor())
        return dest

    def crop(self, upperLeftX, upperLeftY, width, height):
        """Returns a cropped version of this picture

        Copies the pixels in it starting at the specified upper-left corner
        and taking as many pixels as specified by width and height

        Parameters
        ----------
        upperLeftX : int
            the x-coord of the upper-left corner new cropped image
        upperLeftY : int
            the y-coord of the upper-left corner new cropped image
        width : int
            the desired width of the cropped picture
        height : int
            the desired height of the cropped picture

        Returns
        -------
        Picture
            a cropped version of the picture
        """
        croppedImage = self.image.crop((upperLeftX, upperLeftY, upperLeftX+width, upperLeftY+height))
        pic = Picture(croppedImage)
        pic.filename = self.filename
        pic.title = self.title
        return pic

    def scale(self, xFactor, yFactor):
        """Create new scaled picture

        Method to create a new picture by scaling the current picture by
        the given x and y factors

        Parameters
        ----------
        xFactor : float
            the amount to scale in x
        yFactor : float
            the amount to scale in y

        Returns
        -------
        Picture

            a scaled version of the picture
        """
        scaledImage = self.image.resize((int(self.image.width*xFactor), int(self.image.height*yFactor)))
        pic = Picture(scaledImage)
        pic.filename = self.filename
        pic.title = None
        return pic

    def getPictureWithHeight(self, height):
        """Returns a scaled version of this picture

        Scales the picture so that the height is equal to height while keeping
        the same ratio

        Parameters
        ----------
        height : int
            The height of the returned picture

        Returns
        -------
        Picture
            a scaled version of the picture with height of (height)
        """
        # // set up the scale tranform
        yFactor = height / self.getHeight()
        result = self.scale(yFactor, yFactor)
        return result
        
    def getPictureWithWidth(self, width):
        """Returns a scaled version of this picture

        Scales the picture so that the width is equal to width while keeping
        the same ratio

        Parameters
        ----------
        width : int
            The width of the returned picture

        Returns
        -------
        Picture
            a scaled version of the picture with width of (width)
        """
        # // set up the scale tranform
        xFactor = width / self.getWidth()
        result = self.scale(xFactor, xFactor)
        return result

    def loadPictureAndShowIt(self, fileName):
        """Load picture from a file and show it

        Parameters
        ----------
        fileName : str
            the name of the file to load the picture from

        Returns
        -------
        bool
            True if success else False
        """
        result = True

        # try to load the picture into the buffered image from the file name
        result = self.load(fileName)

        # show the picture in a picture frame
        self.show()

        return result

    def load(self, fileName):
        """Load picture from a file without throwing exceptions

        Parameters
        ----------
        fileName : str
            the name of the file to load the picture from

        Returns
        -------
        bool
            True if success else False
        """
        try:
            self.loadOrFail(fileName)
            return True
        except BaseException:
            print("There was an error trying to open " + fileName)
            mode = "RGB"
            size = (600, 200)
            self.image = PIL.Image.new(mode, size, (255,255,255))
            self.addMessage("Couldn't load " + fileName, 5, 100)
            return False

    def loadOrFail(self, fileName):
        """Load a picture from a file

        Parameters
        ----------
        fileName : str
            the name of the file to load the picture from
        """
        self.image = PIL.Image.open(fileName) #.convert('RGB')
        self.filename = self.title = fileName


    def write(self, fileName):
        """Writes this picture to a file with the name fileName

        Parameters
        ----------
        fileName : str
            The name of the file that this picture will be written to

        Returns
        -------
        bool
            True if the file is written False if an IO error occurs
        """
        try :
            self.writeOrFail(fileName)
            return True
        except:
            print("There was an error trying to write " + fileName)
            return False

    def writeOrFail(self, fileName):
        """Write the contents of the picture to a file
 
        Parameters
        ----------
        fileName : str
            the name of the file to write the picture to
        """
        # get name and extension
        name, ext = os.path.splitext(fileName)
        imageType = None
 
        # if no extension, use JES default
        if ext == '':
            imageType = self.extension.replace('.', '')
            if imageType.lower() == 'jpg':
                imageType = 'jpeg'
            print('imageType = {}'.format(imageType))
 
        # write file
        self.image.save(fileName, format=imageType)

    def setMediaPath(self, directory):
        """Method to set the directory for the media

        Parameters
        ----------
        directory : str
            the directory to use for the media path

        """
        FileChooser.setMediaPath(directory)

    def getMediaPath(self, fileName):
        """Method to get the directory for the media

        Parameters
        ----------
        fileName : str
            the fileName the base file name to use

        Returns
        -------
        str
            the full path name by appending the file name to the media directory
        """
        return FileChooser.getMediaPath(fileName)

    def loadImage(self, fileName):
        """Load picture from a file without throwing exceptions

        This just calls load(fileName) and is included for compatibility
        with JES.

        Parameters
        ----------
        fileName : str
            the name of the file to load the picture from

        Returns
        -------
        bool
            True if success else False
        """    
        return self.load(fileName)

    def addMessage(self, message, xPos, yPos):
        """Adds text to the image
    
        message : str
            the message that will be drawn on the picture
        x : int
            the x-coordinate of the top left corner of the text
        y : int
            the y-coordinate of the top left corner of the text
        """
        # get a graphics context to use to draw on the buffered image
        col = Color(0,0,0)
        self.addText(col, xPos, yPos, message)

    def drawString(self, text, xPos, yPos):
        """Adds text to the image
    
        text : str
            the text that will be drawn on the picture
        x : int
            the x-coordinate of the top left corner of the text
        y : int
            the y-coordinate of the top left corner of the text
        """
        # get a graphics context to use to draw on the buffered image
        self.addMessage(text, xPos, yPos)

#----------------------------------------------------------------------------
# WX specific methods
#----------------------------------------------------------------------------

    def getWxImage(self, copy_alpha=True):
        """Return a wx.Image version of this image

        Parameters
        ----------
        copy_alpha : bool
            if True and image has alpha values, then convert them too

        Returns
        -------
        wx.Image
            the converted image
        """
        orig_width, orig_height = self.image.size
        wx_img = wx.Image(orig_width, orig_height)
        wx_img.SetData(self.image.convert('RGB').tobytes())

        if copy_alpha and (self.image.mode[-1] == 'A'):
            alpha = self.image.getchannel("A").tobytes()
            wx_img.InitAlpha()
            for i in range(orig_width):
                for j in range(orig_height):
                    wx_img.SetAlpha(i, j, alpha[i + j * orig_width])
        return wx_img

    def __saveInTempFile(self):
        """Create temporary image file

        Returns
        -------
        string
            path to temporary image file
        """
        filename = os.path.join(tempfile.gettempdir(),
            "jes4py_" + next(tempfile._get_candidate_names()) + self.extension)
        #tempdir = tempfile.mkdtemp()
        #filename = os.path.join(tempdir, "showimage" + self.extension)
        #os.mkdir(tempdir)
        self.write(filename)
        return filename

    def __runScript(self, script, *argv):
        """Run a Python script in a subprocess

        Parameters
        ----------
        script : str
            the script to run; must be in the jes4py directory
        *argv : list
            parameters to pass to script on command line

        Returns
        -------
        Popen instance
        """
        # Start subprocess using current Python intepreter to run a script
        scriptpath = os.path.join(Config.getConfigVal("CONFIG_JES4PY_PATH"),
                script)
        proc = subprocess.Popen([sys.executable, scriptpath] + list(argv),
                stdin=PIPE)

        # Register atexit handler if this is the first subprocess
        if len(self.subprocessList) == 0:
            atexit.register(self.__stopAllSubprocesses)

        # Record the process and return

        self.subprocessList.append(proc)
        return proc

    def __stopAllSubprocesses(self):
        """Close windows (i.e. terminate subprocess)
        """
        for proc in self.subprocessList:
            try:
                self.process.stdin.write(self.show_control_exit)
                self.process.stdin.flush()
                self.process.stdin.close()
                proc.terminate()
                proc.wait(timeout=0.2)
            except: # BrokenPipeError, OSError:
                pass

    def __sendPickledPicture(self):
        """Send pickled self object to "show" process
        """
        pic = Picture(self)
        pkg = pickle.dumps(pic)
        pkgSize = len(pkg).to_bytes(8, byteorder='big')
        self.process.stdin.write(self.show_control_data)
        self.process.stdin.write(pkgSize)
        self.process.stdin.write(pkg)
        self.process.stdin.flush()

    def show(self):
        """Show a picture using stand-alone Python script
        """
        if self.process is None or self.process.poll() is not None:
            # a show process for this pic is not running, start a new one
            self.process = self.__runScript('show.py')
        self.__sendPickledPicture()

    def repaint(self):
        """Reshow a picture using stand-alone Python script
        """
        if (self.process is not None) and self.process.poll() is None:
            # subprocess seems to be running, ask it to update image
            try:
                self.__sendPickledPicture()
            except: # BrokenPipeError:
                # something went wrong, reset and call show
                self.process = None
                self.show()
        else:
            # subprocess is not running, start a new one
            self.process = None
            self.show()

    def pictureTool(self):
        """Explore a picture using a stand-alone Python script
        """
        filename = self.__saveInTempFile()
        self.__runScript('pictureTool.py', filename, self.title)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="jes4py",
    version="0.1.6",
    author="Jonathan Senning",
    author_email="jonathan.senning@gordon.edu",
    description="Python 3 package providing a subset of JES's media functionality",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/gordon-cs/JES4py",
    packages=['jes4py'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6.0',install_requires=[
        'wave',
        'wxPython',
        'simpleaudio'
    ],
    package_data={
        'jes4py': ['images/Left.png','images/Right.png'],
    },

)



#!/usr/bin/env python3

"""
filechooser.py - Script to use wxPython to select a file or folder.

Written: 2021-02-06 Jonathan R. Senning <jonathan.senning@gordon.edu>
Derived from method written by Gahngnin Kim and Jonathan Senning in 2020.
"""

import wx
import sys

def fileDialog(defaultFolder):
    """Use system-dependant file dialog to select a file path.
    
    Lets user pick a file and return the full path as as a string.
    If the user does not pick a file then the path will be None.

    Args:
        defaultFolder (string): initial folder to start search

    Returns:
        the file file name of the picked file or None
    """
    app = wx.App()
    frame = wx.Frame(None, -1, 'PickAFile')

    # Create open file dialog
    path = None
    openFileDialog = wx.FileDialog(frame, 'Pick A File', defaultFolder,
        "", "", wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
    if openFileDialog.ShowModal() == wx.ID_OK:
        path = openFileDialog.GetPath()
    
    # Clean up
    openFileDialog.Destroy()
    return path

def folderDialog(defaultFolder):
    """Use system-dependant file dialog to select a folder path.
    
    Lets user pick a directory and return the full path as a string.
    If the user does not pick a file then the path will be None.

    Args:
        defaultFolder (string): initial folder to start search

    Returns:
        the full directory path
    """
    app = wx.App()
    frame = wx.Frame(None, -1, 'pickADirectory')

    # Create open file dialog
    path = None
    openDirDialog = wx.DirDialog(frame, 'Pick A Folder', defaultFolder,
        wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST)
    if openDirDialog.ShowModal() == wx.ID_OK:
        path = openDirDialog.GetPath()

    # Clean up
    openDirDialog.Destroy()
    return path

def main(argv):
    """Main program.
    """
    if len(argv) == 3:
        path = None
        if argv[1].lower() == 'file':
            path = fileDialog(argv[2])
        elif argv[1].lower() == 'folder':
            path = folderDialog(argv[2])
        if path is not None:
            print(path, end='')
    else:
        print('usage: {} "file"|"folder" start_path'.format(argv[0]))

if __name__ == '__main__':
    main(sys.argv)



from jes4py import Config

class SoundSample:
    """Provides access to frames (samples) within a Sound

    Attributes
    ----------
    wrapLevels : bool
        Indicates whether levels outside the range 0-255 are clamped
        or wrapped around (saturating or modular arithmetic).
        False to clamp levels, true to modulo them.
    """

    wrapLevels = False

    def __init__(self, sound, frameNumber):
        """Constructor

        Parameters
        ----------
        sound : Sound
            sound the frame (sample) belongs to
        frameNumber : int
            index of frame (sample)
        """
        self.wrapLevels = Config.getConfigVal("CONFIG_WRAPPIXELVALUES")
        self.sound = sound
        self.frameNumber = frameNumber

    def __str__(self):
        """Return string with sound sample meta information

        Returns
        -------
        str
            user-readable sound sample information
        """
        return "Sample at index {} has value {}".format(self.frameNumber, self.getValue())

    def __repr__(self):
        """Return string representation of sound sample

        Returns
        -------
        str
            string of sound sample
             contents
        """
        return self.__str__()

    @classmethod
    def correctLevel(cls, level):
        """Map sound level to [-32768,32767] according to wrapLevel setting

        Parameters
        ----------
        level : float
            nonnegative integer representing a color value

        Returns
        -------
        int
            corrected color level
        """
        level = int(level)
        if cls.wrapLevel:
            if level < 0:
                return -1*((-1*level) % 32768)
            elif level > 0:
                return level % 32768
        elif level < -32768:
            return 0
        elif level > 32767:
            return 32767
        return level

    @classmethod
    def setWrapLevels(cls, wrap):
        """Round and correct a sound level to be within (-32768, 32767)

        Parameters
        ----------
        wrap : bool
            True to enable level wrapping, False for level truncation
        """
        cls.wrapLevels = wrap

    @classmethod
    def getWrapLevels(cls):
        """Return current setting of wrap level

        Indicates whether levels outside the range (-32768, 32767) are
        clamped or wrapped around (saturating or modular arithmetic).

        Returns
        -------
        bool
            True means levels are wrapped, False means levels are truncated
        """
        return cls.wrapLevels

    def getValue(self):
        value = 0
        try:
            value = self.sound.getSampleValue(self.frameNumber)
        except:
            pass
        return value
    
    def setValue(self, value):
        try:
            self.sound.setSampleValue(self.frameNumber, value)
        except:
            pass



#!/usr/bin/env python3

"""
colorchooser.py - Script to use wxPython to allow user to select color.

Written: 2021-02-06 Jonathan R. Senning <jonathan.senning@gordon.edu>
Derived from method written by Gahngnin Kim and Jonathan Senning in 2020.
"""

import wx

def chooseColor():
    """Use system-dependent color chooser dialog to select color.

    Returns:
        RGB tuple or None.
    """
    app = wx.App()

    # setup defaults
    data = wx.ColourData()
    if wx.Platform == "__WXMAC__":
        data.SetChooseAlpha(False)
    data.SetChooseFull(True)
    data.SetColour(wx.WHITE) # show colors on initial color wheel on mac

    # start and process the dialog
    color = None
    dlg = wx.ColourDialog(wx.GetApp().GetTopWindow(), data)
    if dlg.ShowModal() == wx.ID_OK:
        red   = dlg.GetColourData().GetColour().Red()
        green = dlg.GetColourData().GetColour().Green()
        blue  = dlg.GetColourData().GetColour().Blue()
        color = (red, green, blue)

    # clean up
    dlg.Destroy()
    return color

if __name__ == '__main__':
    color = chooseColor()
    if color is not None:
        print(color[0], color[1], color[2], end='')
